import sys
import math
import pytest
from sdl2 import endian


class TestSDLEndian(object):
    __tags__ = ["sdl"]

    def test_SDL_BYTEORDER(self):
        if sys.byteorder == "little":
            assert endian.SDL_BYTEORDER == endian.SDL_LIL_ENDIAN
        else:
            assert endian.SDL_BYTEORDER == endian.SDL_BIG_ENDIAN

    def test_SDL_Swap16(self):
        assert endian.SDL_Swap16(0xFF00) == 0x00FF
        assert endian.SDL_Swap16(0x0001) == 0x0100
        assert endian.SDL_Swap16(0x0032) == 0x3200
        assert endian.SDL_Swap16(0x0FF0) == 0xF00F
        assert endian.SDL_Swap16(0x00FF) == 0xFF00
        assert endian.SDL_Swap16(0x1234) == 0x3412
        if sys.byteorder == "little":
            assert endian.SDL_Swap16 == endian.SDL_SwapBE16
            assert endian.SDL_Swap16 != endian.SDL_SwapLE16
        else:
            assert endian.SDL_Swap16 != endian.SDL_SwapBE16
            assert endian.SDL_Swap16 == endian.SDL_SwapLE16

    def test_SDL_Swap32(self):
        assert endian.SDL_Swap32(0xFF000000) == 0x000000FF
        assert endian.SDL_Swap32(0x00FF0000) == 0x0000FF00
        assert endian.SDL_Swap32(0x0000FF00) == 0x00FF0000
        assert endian.SDL_Swap32(0x000000FF) == 0xFF000000
        assert endian.SDL_Swap32(0x12345678) == 0x78563412
        assert endian.SDL_Swap32(0xFF00FF00) == 0x00FF00FF
        if sys.byteorder == "little":
            assert endian.SDL_Swap32 == endian.SDL_SwapBE32
            assert endian.SDL_Swap32 != endian.SDL_SwapLE32
        else:
            assert endian.SDL_Swap32 != endian.SDL_SwapBE32
            assert endian.SDL_Swap32 == endian.SDL_SwapLE32

    def test_SDL_Swap64(self):
        assert endian.SDL_Swap64(0xFF00000000000000) == 0x00000000000000FF
        assert endian.SDL_Swap64(0x00FF000000000000) == 0x000000000000FF00
        assert endian.SDL_Swap64(0x0000FF0000000000) == 0x0000000000FF0000
        assert endian.SDL_Swap64(0x000000FF00000000) == 0x00000000FF000000
        assert endian.SDL_Swap64(0x00000000FF000000) == 0x000000FF00000000
        assert endian.SDL_Swap64(0x0000000000FF0000) == 0x0000FF0000000000
        assert endian.SDL_Swap64(0x000000000000FF00) == 0x00FF000000000000
        assert endian.SDL_Swap64(0x00000000000000FF) == 0xFF00000000000000
        assert endian.SDL_Swap64(0x0123456789ABCDEF) == 0xEFCDAB8967452301
        if sys.byteorder == "little":
            assert endian.SDL_Swap64 == endian.SDL_SwapBE64
            assert endian.SDL_Swap64 != endian.SDL_SwapLE64
        else:
            assert endian.SDL_Swap64 != endian.SDL_SwapBE64
            assert endian.SDL_Swap64 == endian.SDL_SwapLE64

    def test_SDL_SwapFloat(self):
        v = -100.0
        while v < 101:
            p = endian.SDL_SwapFloat(v)
            assert p != v
            assert endian.SDL_SwapFloat(p) == v
            v += 0.1
        values = (sys.float_info.epsilon,
                  sys.float_info.min,
                  sys.float_info.max,
                  - sys.float_info.min,
                  math.pi,
                  - math.pi
                  )
        for v in values:
            p = endian.SDL_SwapFloat(v)
            assert p != v
            assert endian.SDL_SwapFloat(p) == v

        if sys.byteorder == "little":
            assert endian.SDL_SwapFloat == endian.SDL_SwapFloatBE
            assert endian.SDL_SwapFloat != endian.SDL_SwapFloatLE
        else:
            assert endian.SDL_SwapFloat != endian.SDL_SwapFloatBE
            assert endian.SDL_SwapFloat == endian.SDL_SwapFloatLE



import sys
import pytest
import ctypes
from sdl2 import SDL_Init, SDL_Quit, SDL_QuitSubSystem, SDL_INIT_EVERYTHING
from sdl2 import video, shape, surface


class TestSDLShape(object):
    __tags__ = ["sdl"]

    @classmethod
    def setup_class(cls):
        SDL_Init(SDL_INIT_EVERYTHING)
        if video.SDL_GetCurrentVideoDriver() is None:
            raise pytest.skip('Video subsystem not supported')

    @classmethod
    def teardown_class(cls):
        SDL_QuitSubSystem(SDL_INIT_EVERYTHING)
        SDL_Quit()

    def test_SDL_CreateShapedWindow(self):
        if video.SDL_GetCurrentVideoDriver() == b"dummy":
            pytest.skip("dummy video driver does not support shaped windows")
        flags = (video.SDL_WINDOW_HIDDEN,)
        for flag in flags:
            window = shape.SDL_CreateShapedWindow(b"Test", 10, 10, 10, 10,
                                                  flag)
            assert isinstance(window.contents, video.SDL_Window)
            video.SDL_DestroyWindow(window)

    def test_SDL_IsShapedWindow(self):
        if video.SDL_GetCurrentVideoDriver() == b"dummy":
            pytest.skip("dummy video driver does not support shaped windows")
        flags = (video.SDL_WINDOW_HIDDEN,)
        for flag in flags:
            window = shape.SDL_CreateShapedWindow(b"Test", 10, 10, 10, 10,
                                                  flag)
            assert isinstance(window.contents, video.SDL_Window)
            val = shape.SDL_IsShapedWindow(window)
            assert val
            video.SDL_DestroyWindow(window)

            window = video.SDL_CreateWindow(b"Test", 10, 10, 10, 10, flag)
            assert isinstance(window.contents, video.SDL_Window)
            val = shape.SDL_IsShapedWindow(window)
            assert not val
            video.SDL_DestroyWindow(window)

    def test_SDL_SetWindowShape(self):
        if video.SDL_GetCurrentVideoDriver() == b"dummy":
            pytest.skip("dummy video driver does not support shaped windows")
        sf = surface.SDL_CreateRGBSurface(0, 10, 10, 32,
                                          0xFF000000,
                                          0x00FF0000,
                                          0x0000FF00,
                                          0x000000FF)
        mode = shape.SDL_WindowShapeMode()
        mode.mode = shape.ShapeModeDefault
        mode.parameters = shape.SDL_WindowShapeParams()
        mode.parameters.binarizationCutoff = 1

        flags = (video.SDL_WINDOW_HIDDEN,)
        for flag in flags:
            # TODO: find out, how shaping is supposed to work :-)
            # window = shape.SDL_CreateShapedWindow(b"Test", 10, 10, 10, 10,
            #                                       flag)
            # self.assertIsInstance(window, video.SDL_Window)
            # self.assertTrue(shape.SDL_IsShapedWindow(window))

            # self.assertRaises(TypeError, shape.SDL_SetWindowShape,
            #                   None, None, None)
            # self.assertRaises(TypeError, shape.SDL_SetWindowShape,
            #                   window, None, None)
            # self.assertRaises(TypeError, shape.SDL_SetWindowShape,
            #                   window, sf, None)
            # self.assertRaises(TypeError, shape.SDL_SetWindowShape,
            #                   "Test", sf, mode)
            # self.assertRaises(TypeError, shape.SDL_SetWindowShape,
            #                   window, 12345, mode)
            # self.assertRaises(TypeError, shape.SDL_SetWindowShape,
            #                   window, sf, "Test")

            # shape.SDL_SetWindowShape(window, sf, mode)
            # wmode = shape.SDL_GetShapedWindowMode(window)
            # self.assertEqual(wmode.mode, mode.mode)
            # self.assertEqual(wmode.parameters.binarizationCutoff,
            #                  mode.parameters.binarizationCutoff)
            # video.SDL_DestroyWindow(window)

            window = video.SDL_CreateWindow(b"Test", 10, 10, 10, 10, flag)
            assert isinstance(window.contents, video.SDL_Window)
            #self.assertRaises(sdl.SDLError, shape.SDL_SetWindowShape,
            #                  window, sf, mode)
            video.SDL_DestroyWindow(window)
        surface.SDL_FreeSurface(sf)

    def test_SDL_GetShapedWindowMode(self):
        if video.SDL_GetCurrentVideoDriver() == b"dummy":
            pytest.skip("dummy video driver does not support shaped windows")
        flags = (video.SDL_WINDOW_HIDDEN,)
        for flag in flags:
            window = shape.SDL_CreateShapedWindow(b"Test", 10, 10, 10, 10,
                                                  flag)
            assert isinstance(window.contents, video.SDL_Window)
            mode = shape.SDL_WindowShapeMode()
            ret = shape.SDL_GetShapedWindowMode(window, ctypes.byref(mode))
            assert ret == 0
            video.SDL_DestroyWindow(window)



import sys
import pytest
import copy
from sdl2.ext import color
from sdl2.ext.color import Color
from sdl2.ext.compat import *

combs = [0, 1, 2, 4, 8, 16, 32, 62, 63, 64, 126, 127, 128, 255]
all_combos = [(r, g, b, a) for r in combs
                           for g in combs
                           for b in combs
                           for a in combs]


def color_combos():
    for x in all_combos:
        yield Color(*x)


def hex_combos():
    for r, g, b, a in all_combos:
        yield "0x%.2x%.2x%.2x%.2x" % (r, g, b, a)


def hash_combos():
    for r, g, b, a in all_combos:
        yield "#%.2x%.2x%.2x%.2x" % (r, g, b, a)


def rgba_combos():
    for r, g, b, a in all_combos:
        yield (r << 24 | g << 16 | b << 8 | a)


def argb_combos():
    for r, g, b, a in all_combos:
        yield (a << 24 | r << 16 | g << 8 | b)


def _assignr(x, y):
    x.r = y


def _assigng(x, y):
    x.g = y


def _assignb(x, y):
    x.b = y


def _assigna(x, y):
    x.a = y


def _assign_item(x, p, y):
    x[p] = y


class TestSDL2ExtColor(object):
    __tags__ = ["sdl2ext"]

    def test_Color(self):
        c = Color(10, 20, 30, 40)
        assert c.r == 10
        assert c.g == 20
        assert c.b == 30
        assert c.a == 40

        with pytest.raises(ValueError):
            Color(257, 10, 105, 44)
        with pytest.raises(ValueError):
            Color(10, 257, 105, 44)
        with pytest.raises(ValueError):
            Color(10, 105, 257, 44)
        with pytest.raises(ValueError):
            Color(10, 105, 44, 257)


    def test_Color__copy__(self):
        copy_copy = copy.copy
        for c in color_combos():
            c2 = copy_copy(c)
            assert c == c2
            c2 = ~c2
            assert c != c2


    def test_Color__eq__(self):
        assert Color(255, 0, 0, 0) == Color(255, 0, 0, 0)
        assert Color(0, 255, 0, 0) == Color(0, 255, 0, 0)
        assert Color(0, 0, 255, 0) == Color(0, 0, 255, 0)
        assert Color(0, 0, 0, 255) == Color(0, 0, 0, 255)

        assert not (Color(0, 0, 0, 0) == Color(255, 0, 0, 0))
        assert not (Color(0, 0, 0, 0) == Color(0, 255, 0, 0))
        assert not (Color(0, 0, 0, 0) == Color(0, 0, 255, 0))
        assert not (Color(0, 0, 0, 0) == Color(0, 0, 0, 255))

        assert tuple(Color(255, 0, 0, 0)) == (255, 0, 0, 0)
        assert tuple(Color(0, 255, 0, 0)) == (0, 255, 0, 0)
        assert tuple(Color(0, 0, 255, 0)) == (0, 0, 255, 0)
        assert tuple(Color(0, 0, 0, 255)) == (0, 0, 0, 255)

        assert not (tuple(Color(0, 0, 0, 0)) == (255, 0, 0, 0))
        assert not (tuple(Color(0, 0, 0, 0)) == (0, 255, 0, 0))
        assert not (tuple(Color(0, 0, 0, 0)) == (0, 0, 255, 0))
        assert not (tuple(Color(0, 0, 0, 0)) == (0, 0, 0, 255))

        assert int(Color(255, 0, 0, 0)) == 0xff000000
        assert int(Color(0, 255, 0, 0)) == 0x00ff0000
        assert int(Color(0, 0, 255, 0)) == 0x0000ff00
        assert int(Color(0, 0, 0, 255)) == 0x000000ff

        assert not (int(Color(0, 0, 0, 0)) == 0xff000000)
        assert not (int(Color(0, 0, 0, 0)) == 0x00ff0000)
        assert not (int(Color(0, 0, 0, 0)) == 0x0000ff00)
        assert not (int(Color(0, 0, 0, 0)) == 0x000000ff)


    def test_Color__ne__(self):
        assert Color(0, 0, 0, 0) != Color(255, 0, 0, 0)
        assert Color(0, 0, 0, 0) != Color(0, 255, 0, 0)
        assert Color(0, 0, 0, 0) != Color(0, 0, 255, 0)
        assert Color(0, 0, 0, 0) != Color(0, 0, 0, 255)

        assert not (Color(255, 0, 0, 0) != Color(255, 0, 0, 0))
        assert not (Color(0, 255, 0, 0) != Color(0, 255, 0, 0))
        assert not (Color(0, 0, 255, 0) != Color(0, 0, 255, 0))
        assert not (Color(0, 0, 0, 255) != Color(0, 0, 0, 255))

        assert tuple(Color(0, 0, 0, 0)) != (255, 0, 0, 0)
        assert tuple(Color(0, 0, 0, 0)) != (0, 255, 0, 0)
        assert tuple(Color(0, 0, 0, 0)) != (0, 0, 255, 0)
        assert tuple(Color(0, 0, 0, 0)) != (0, 0, 0, 255)

        assert not (tuple(Color(255, 0, 0, 0)) != (255, 0, 0, 0))
        assert not (tuple(Color(0, 255, 0, 0)) != (0, 255, 0, 0))
        assert not (tuple(Color(0, 0, 255, 0)) != (0, 0, 255, 0))
        assert not (tuple(Color(0, 0, 0, 255)) != (0, 0, 0, 255))

        assert int(Color(0, 0, 0, 0)) != 0xff000000
        assert int(Color(0, 0, 0, 0)) != 0x00ff0000
        assert int(Color(0, 0, 0, 0)) != 0x0000ff00
        assert int(Color(0, 0, 0, 0)) != 0x000000ff

        assert not (int(Color(255, 0, 0, 0)) != 0xff000000)
        assert not (int(Color(0, 255, 0, 0)) != 0x00ff0000)
        assert not (int(Color(0, 0, 255, 0)) != 0x0000ff00)
        assert not (int(Color(0, 0, 0, 255)) != 0x000000ff)


    def test_Color__repr__(self):
        c = Color(68, 38, 26, 69)
        c1 = eval(repr(c))
        assert c == c1


    def test_Color__int__(self):
        c = Color(0x00, 0xCC, 0x00, 0xCC)
        assert c.r == 0x00
        assert c.g == 0xCC
        assert c.b == 0x00
        assert c.a == 0xCC
        assert int(c) == int(0x00CC00CC)

        c = Color(0x72, 0x75, 0x92, 0x33)
        assert c.r == 0x72
        assert c.g == 0x75
        assert c.b == 0x92
        assert c.a == 0x33
        assert int(c) == int(0x72759233)


    def test_Color__long__(self):
        c = Color(0x00, 0xCC, 0x00, 0xCC)
        assert c.r == 0x00
        assert c.g == 0xCC
        assert c.b == 0x00
        assert c.a == 0xCC
        assert long(c) == long(0x00CC00CC)

        c = Color(0x72, 0x75, 0x92, 0x33)
        assert c.r == 0x72
        assert c.g == 0x75
        assert c.b == 0x92
        assert c.a == 0x33
        assert long(c) == long(0x72759233)


    def test_Color__float__(self):
        c = Color(0x00, 0xCC, 0x00, 0xCC)
        assert c.r == 0x00
        assert c.g == 0xCC
        assert c.b == 0x00
        assert c.a == 0xCC
        assert float(c) == float(0x00CC00CC)

        c = Color(0x72, 0x75, 0x92, 0x33)
        assert c.r == 0x72
        assert c.g == 0x75
        assert c.b == 0x92
        assert c.a == 0x33
        assert float(c) == float(0x72759233)


    def test_Color__oct__(self):
        c = Color(0x00, 0xCC, 0x00, 0xCC)
        assert c.r == 0x00
        assert c.g == 0xCC
        assert c.b == 0x00
        assert c.a == 0xCC
        assert oct(c) == oct(0x00CC00CC)

        c = Color(0x72, 0x75, 0x92, 0x33)
        assert c.r == 0x72
        assert c.g == 0x75
        assert c.b == 0x92
        assert c.a == 0x33
        assert oct(c) == oct(0x72759233)


    def test_Color__hex__(self):
        c = Color(0x00, 0xCC, 0x00, 0xCC)
        assert c.r == 0x00
        assert c.g == 0xCC
        assert c.b == 0x00
        assert c.a == 0xCC
        assert hex(c) == hex(0x00CC00CC)

        c = Color(0x72, 0x75, 0x92, 0x33)
        assert c.r == 0x72
        assert c.g == 0x75
        assert c.b == 0x92
        assert c.a == 0x33
        assert hex(c) == hex(0x72759233)


    def test_Color__invert__(self):
        assert ~Color() == Color(0, 0, 0, 0)
        assert ~Color(0, 0, 0, 0) == Color(255, 255, 255, 255)
        assert ~Color(255, 0, 0, 0) == Color(0, 255, 255, 255)
        assert ~Color(0, 255, 0, 0) == Color(255, 0, 255, 255)
        assert ~Color(0, 0, 255, 0) == Color(255, 255, 0, 255)
        assert ~Color(0, 0, 0, 255) == Color(255, 255, 255, 0)
        assert ~Color(127, 127, 127, 0) == Color(128, 128, 128, 255)

        assert ~Color(1, 2, 3, 4) == Color(254, 253, 252, 251)
        assert ~Color(127, 127, 127, 0) == Color(128, 128, 128, 255)


    def test_Color__mod__(self):
        c1 = Color()
        assert c1.r == 255
        assert c1.g == 255
        assert c1.b == 255
        assert c1.a == 255

        c2 = Color(2, 4, 8, 16)
        assert c2.r == 2
        assert c2.g == 4
        assert c2.b == 8
        assert c2.a == 16

        c3 = c1 % c2
        assert c3.r == 1
        assert c3.g == 3
        assert c3.b == 7
        assert c3.a == 15


    def test_Color__div__(self):
        c1 = Color(128, 128, 128, 128)
        assert c1.r == 128
        assert c1.g == 128
        assert c1.b == 128
        assert c1.a == 128

        c2 = Color(2, 4, 8, 16)
        assert c2.r == 2
        assert c2.g == 4
        assert c2.b == 8
        assert c2.a == 16

        c3 = c1 / c2
        assert c3.r == 64
        assert c3.g == 32
        assert c3.b == 16
        assert c3.a == 8

        c3 = c3 / c2
        assert c3.r == 32
        assert c3.g == 8
        assert c3.b == 2
        assert c3.a == 0


    def test_Color__mul__(self):
        c1 = Color(1, 1, 1, 1)
        assert c1.r == 1
        assert c1.g == 1
        assert c1.b == 1
        assert c1.a == 1

        c2 = Color(2, 5, 3, 22)
        assert c2.r == 2
        assert c2.g == 5
        assert c2.b == 3
        assert c2.a == 22

        c3 = c1 * c2
        assert c3.r == 2
        assert c3.g == 5
        assert c3.b == 3
        assert c3.a == 22

        c3 = c3 * c2
        assert c3.r == 4
        assert c3.g == 25
        assert c3.b == 9
        assert c3.a == 255


    def test_Color__sub__(self):
        c1 = Color(255, 255, 255, 255)
        assert c1.r == 255
        assert c1.g == 255
        assert c1.b == 255
        assert c1.a == 255

        c2 = Color(20, 33, 82, 193)
        assert c2.r == 20
        assert c2.g == 33
        assert c2.b == 82
        assert c2.a == 193

        c3 = c1 - c2
        assert c3.r == 235
        assert c3.g == 222
        assert c3.b == 173
        assert c3.a == 62

        c3 = c3 - c2
        assert c3.r == 215
        assert c3.g == 189
        assert c3.b == 91
        assert c3.a == 0


    def test_Color__add__(self):
        c1 = Color(0, 0, 0, 0)
        assert c1.r == 0
        assert c1.g == 0
        assert c1.b == 0
        assert c1.a == 0

        c2 = Color(20, 33, 82, 193)
        assert c2.r == 20
        assert c2.g == 33
        assert c2.b == 82
        assert c2.a == 193

        c3 = c1 + c2
        assert c3.r == 20
        assert c3.g == 33
        assert c3.b == 82
        assert c3.a == 193

        c3 = c3 + c2
        assert c3.r == 40
        assert c3.g == 66
        assert c3.b == 164
        assert c3.a == 255


    def test_Color__len__(self):
        c = Color(204, 38, 194, 55)
        assert len(c) == 4
        assert len(Color()) == 4
        assert len(Color(2)) == 4


    def test_Color__getitem__(self):
        c = Color(204, 38, 194, 55)
        assert c[0] == 204
        assert c[1] == 38
        assert c[2] == 194
        assert c[3] == 55


    def test_Color__setitem(self):
        c = Color(204, 38, 194, 55)
        assert c[0] == 204
        assert c[1] == 38
        assert c[2] == 194
        assert c[3] == 55

        c[0] = 33
        assert c[0] == 33
        c[1] = 48
        assert c[1] == 48
        c[2] = 173
        assert c[2] == 173
        c[3] = 213
        assert c[3] == 213

        # Now try some 'invalid' ones
        with pytest.raises(ValueError):
            _assign_item(c, 1, -83)
        assert c[1] == 48
        with pytest.raises(TypeError):
            _assign_item(c, 2, "Hello")
        assert c[2] == 173


    def test_Color_r(self):
        c = Color(100, 100, 100)
        assert c.r == 100

        c = Color(100, 100, 100, 100)
        assert c.r == 100

        c = Color(100, 100, 100)
        assert c.r == 100
        c.r = 200
        assert c.r == 200
        c.r += 22
        assert c.r == 222


    def test_Color_g(self):
        c = Color(100, 100, 100)
        assert c.g == 100

        c = Color(100, 100, 100, 100)
        assert c.g == 100

        c = Color(100, 100, 100)
        assert c.g == 100
        c.g = 200
        assert c.g == 200
        c.g += 22
        assert c.g == 222


    def test_Color_b(self):
        c = Color(100, 100, 100)
        assert c.b == 100

        c = Color(100, 100, 100, 100)
        assert c.b == 100

        c = Color(100, 100, 100)
        assert c.b == 100
        c.b = 200
        assert c.b == 200
        c.b += 22
        assert c.b == 222


    def test_Color_a(self):
        c = Color(100, 100, 100)
        assert c.a == 255

        c = Color(100, 100, 100, 100)
        assert c.a == 100

        c = Color(100, 100, 100)
        assert c.a == 255
        c.a = 200
        assert c.a == 200
        c.a += 22
        assert c.a == 222


    def test_Color_rgba(self):
        c = Color(0)
        assert c.r == 0
        assert c.g == 255
        assert c.b == 255
        assert c.a == 255

        # Test simple assignments
        c.r = 123
        assert c.r == 123
        with pytest.raises(ValueError):
            _assignr(c, 537)
        assert c.r == 123
        with pytest.raises(ValueError):
            _assignr(c, -3)
        assert c.r == 123
        assert c.g == 255
        assert c.b == 255
        assert c.a == 255

        c.g = 55
        assert c.g == 55
        with pytest.raises(ValueError):
            _assigng(c, 348)
        assert c.g == 55
        with pytest.raises(ValueError):
            _assigng(c, -44)
        assert c.g == 55
        assert c.r == 123
        assert c.b == 255
        assert c.a == 255

        c.b = 77
        assert c.b == 77
        with pytest.raises(ValueError):
            _assignb(c, 256)
        assert c.b == 77
        with pytest.raises(ValueError):
            _assignb(c, -12)
        assert c.b == 77
        assert c.r == 123
        assert c.g == 55
        assert c.a == 255

        c.a = 251
        assert c.a == 251
        with pytest.raises(ValueError):
            _assigna(c, 312)
        assert c.a == 251
        with pytest.raises(ValueError):
            _assigna(c, -10)
        assert c.a == 251
        assert c.r == 123
        assert c.g == 55
        assert c.b == 77


    def test_Color_hsva(self):
        for c in color_combos():
            h, s, v, a = c.hsva
            assert 0 <= h <= 360
            assert 0 <= s <= 100
            assert 0 <= v <= 100
            assert 0 <= a <= 100

            for cx in [(0, 0, 0, 0), (255, 255, 255, 255)]:
                c2 = Color(*cx)
                assert tuple(c2) == cx

                c2.hsva = c.hsva
                err = "Failed for color '%s' and cx '%s': %s" % (c, cx, c2)
                assert abs(c2.r - c.r <= 1), err
                assert abs(c2.g - c.g <= 1), err
                assert abs(c2.b - c.b <= 1), err
                assert abs(c2.a - c.a <= 1), err


    def test_Color_hsla(self):
        for c in color_combos():
            h, s, l, a = c.hsla
            assert 0 <= h <= 360
            assert 0 <= s <= 100
            assert 0 <= l <= 100
            assert 0 <= a <= 100

            for cx in [(0, 0, 0, 0), (255, 255, 255, 255)]:
                c2 = Color(*cx)
                assert tuple(c2) == cx

                c2.hsla = c.hsla
                err = "Failed for color '%s' and cx '%s': %s" % (c, cx, c2)
                assert abs(c2.r - c.r <= 1), err
                assert abs(c2.g - c.g <= 1), err
                assert abs(c2.b - c.b <= 1), err
                assert abs(c2.a - c.a <= 1), err


    def test_Color_i1i2i3(self):
        for c in color_combos():
            i1, i2, i3 = c.i1i2i3
            assert 0 <= i1 <= 1
            assert -0.5 <= i2 <= 0.5
            assert -0.5 <= i3 <= 0.5

            for cx in [(0, 0, 0, 0), (255, 255, 255, 255)]:
                c2 = Color(*cx)
                assert tuple(c2) == cx

                c2.i1i2i3 = c.i1i2i3
                err = "Failed for color '%s' and cx '%s': %s" % (c, cx, c2)
                # I1I2I3 ignores the alpha channel, thus we won't check it
                assert abs(c2.r - c.r <= 1), err
                assert abs(c2.g - c.g <= 1), err
                assert abs(c2.b - c.b <= 1), err


    def test_Color_cmy(self):
        for val in color_combos():
            c, m, y = val.cmy
            assert 0 <= c <= 1
            assert 0 <= m <= 1
            assert 0 <= y <= 1

            for cx in [(0, 0, 0, 0), (255, 255, 255, 255)]:
                c2 = Color(*cx)
                assert tuple(c2) == cx

                c2.cmy = val.cmy
                err = "Failed for color '%s' and cx '%s': %s" % (c, cx, c2)
                # CMY ignores the alpha channel, thus we won't check it
                assert abs(c2.r - val.r <= 1), err
                assert abs(c2.g - val.g <= 1), err
                assert abs(c2.b - val.b <= 1), err


    def test_Color_normalize(self):
        c = Color(204, 38, 194, 55)
        assert c.r == 204
        assert c.g == 38
        assert c.b == 194
        assert c.a == 55

        t = c.normalize()

        assert round(abs(t[0]-0.800000), 5) == 0
        assert round(abs(t[1]-0.149016), 5) == 0
        assert round(abs(t[2]-0.760784), 5) == 0
        assert round(abs(t[3]-0.215686), 5) == 0

        c = Color(255, 255, 255, 255)
        assert c.normalize() == (1.0, 1.0, 1.0, 1.0)
        c = Color(0, 0, 0, 0)
        assert c.normalize() == (0.0, 0.0, 0.0, 0.0)
        c = Color(128, 128, 128, 128)
        t = c.normalize()
        for v in t:
            assert round(abs(v-0.5), 2) == 0

        c = Color(128, 255, 0, 52)
        t = c.normalize()
        assert round(abs(t[0]-0.5), 2) == 0
        assert t[1] == 1.0
        assert t[2] == 0.0
        # 52 / 255 ~= .20
        assert round(abs(t[3]-0.2), 2) == 0


    def test_is_rgb_color(self):
        for v in color_combos():
            assert color.is_rgba_color(v)

        for v in rgba_combos():
            assert not color.is_rgba_color(v)
        for v in argb_combos():
            assert not color.is_rgba_color(v)
        for v in hex_combos():
            assert not color.is_rgba_color(v)
        for v in hash_combos():
            assert not color.is_rgba_color(v)


    def test_is_rgba_color(self):
        for v in color_combos():
            assert color.is_rgba_color(v)

        for v in rgba_combos():
            assert not color.is_rgba_color(v)
        for v in argb_combos():
            assert not color.is_rgba_color(v)
        for v in hex_combos():
            assert not color.is_rgba_color(v)
        for v in hash_combos():
            assert not color.is_rgba_color(v)


    def test_rgba_argb_to_color(self):

        assert color.RGBA == color.rgba_to_color
        assert color.ARGB == color.argb_to_color

        cvals = list(color_combos())
        for index, val in enumerate(rgba_combos()):
            c = cvals[index]
            if c.r == c.g == c.b == c.a:
                assert color.RGBA(val) == c
                assert color.ARGB(val) == c
                continue

            assert color.RGBA(val) == c, "Failed for '%s'" % val
            assert not color.ARGB(val) == c, "Failed for '0x%.8x'" % val

        for index, val in enumerate(argb_combos()):
            c = cvals[index]
            if c.r == c.g == c.b == c.a:
                assert color.RGBA(val) == c
                assert color.ARGB(val) == c
                continue

            assert color.ARGB(val) == c, "Failed for '%s'" % val
            assert not color.RGBA(val) == c, "Failed for '0x%.8x'" % val


    def test_string_to_color(self):
        for method in (color.string_to_color, color.convert_to_color,
                       color.COLOR):
            assert method('#00000000').r == 0x00
            assert method('#10000000').r == 0x10
            assert method('#20000000').r == 0x20
            assert method('#30000000').r == 0x30
            assert method('#40000000').r == 0x40
            assert method('#50000000').r == 0x50
            assert method('#60000000').r == 0x60
            assert method('#70000000').r == 0x70
            assert method('#80000000').r == 0x80
            assert method('#90000000').r == 0x90
            assert method('#A0000000').r == 0xA0
            assert method('#B0000000').r == 0xB0
            assert method('#C0000000').r == 0xC0
            assert method('#D0000000').r == 0xD0
            assert method('#E0000000').r == 0xE0
            assert method('#F0000000').r == 0xF0
            assert method('#01000000').r == 0x01
            assert method('#02000000').r == 0x02
            assert method('#03000000').r == 0x03
            assert method('#04000000').r == 0x04
            assert method('#05000000').r == 0x05
            assert method('#06000000').r == 0x06
            assert method('#07000000').r == 0x07
            assert method('#08000000').r == 0x08
            assert method('#09000000').r == 0x09
            assert method('#0A000000').r == 0x0A
            assert method('#0B000000').r == 0x0B
            assert method('#0C000000').r == 0x0C
            assert method('#0D000000').r == 0x0D
            assert method('#0E000000').r == 0x0E
            assert method('#0F000000').r == 0x0F

            with pytest.raises(ValueError):
                method("0x12345")
            with pytest.raises(ValueError):
                method("0x1234567")
            with pytest.raises(ValueError):
                method("#123456789")
            with pytest.raises(ValueError):
                method("#12345")
            with pytest.raises(ValueError):
                method("#1234567")
            with pytest.raises(ValueError):
                method("#123456789")

            with pytest.raises(ValueError):
                method("# f000000")
            with pytest.raises(ValueError):
                method("#f 000000")
            with pytest.raises(ValueError):
                method("#-f000000")
            with pytest.raises(ValueError):
                method("-#f000000")

            with pytest.raises(ValueError):
                method("0x f000000")
            with pytest.raises(ValueError):
                method("0xf 000000")
            with pytest.raises(ValueError):
                method("0x-f000000")
            with pytest.raises(ValueError):
                method("-0xf000000")

            with pytest.raises(ValueError):
                method("#cc00qq")
            with pytest.raises(ValueError):
                method("0xcc00qq")
            with pytest.raises(ValueError):
                method("09abcdef")
            with pytest.raises(ValueError):
                method("09abcde")
            with pytest.raises(ValueError):
                method("quarky")

            cvals = list(color_combos())
            for index, val in enumerate(hex_combos()):
                assert method(val) == cvals[index], "Failed for '%s'" % val
            for index, val in enumerate(hash_combos()):
                assert method(val) == cvals[index], "Failed for '%s'" % val

        with pytest.raises(TypeError):
            color.string_to_color(0xff000000)
        with pytest.raises(TypeError):
            color.string_to_color(Color())


    def test_convert_to_color(self):
        assert color.COLOR == color.convert_to_color
        cvals = list(color_combos())

        for index, val in enumerate(hex_combos()):
            assert color.COLOR(val) == cvals[index], "Failed for '%s'" % val

        for index, val in enumerate(hash_combos()):
            assert color.COLOR(val) == cvals[index], "Failed for '%s'" % val

        for index, val in enumerate(hex_combos()):
            assert color.COLOR(val) == cvals[index], "Failed for '%s'" % val

        for index, val in enumerate(argb_combos()):
            assert color.COLOR(val) == cvals[index], "Failed for '0x%.8x'" % val

        for index, val in enumerate(color_combos()):
            assert color.COLOR(val) == cvals[index], "Failed for '%s'" % val

        with pytest.raises(ValueError):
            color.convert_to_color(self)

        with pytest.raises(ValueError):
            color.convert_to_color("Test")



import sys
import pytest
import copy
from ctypes import c_int, POINTER, byref, cast, ArgumentError
from sdl2 import SDL_Init, SDL_Quit, SDL_QuitSubSystem, SDL_INIT_EVERYTHING
from sdl2 import pixels
from sdl2.pixels import SDL_Color
from sdl2.stdinc import Uint8, Uint16, Uint32



class TestSDLPixels(object):
    __tags__ = ["sdl"]

    @classmethod
    def setup_class(cls):
        SDL_Init(SDL_INIT_EVERYTHING)

    @classmethod
    def teardown_class(cls):
        SDL_QuitSubSystem(SDL_INIT_EVERYTHING)
        SDL_Quit()

    def test_SDL_FOURCC(self):
        with pytest.raises(TypeError):
            pixels.SDL_FOURCC(None, None, None, None)
        with pytest.raises(TypeError):
            pixels.SDL_FOURCC("a", None, None, None)
        with pytest.raises(TypeError):
            pixels.SDL_FOURCC(None, "a", None, None)
        with pytest.raises(TypeError):
            pixels.SDL_FOURCC(None, None, "a", None)
        with pytest.raises(TypeError):
            pixels.SDL_FOURCC(None, None, None, "a")
        with pytest.raises(TypeError):
            pixels.SDL_FOURCC("a", "a", None, None)
        with pytest.raises(TypeError):
            pixels.SDL_FOURCC("a", "a", "a", None)
        with pytest.raises(TypeError):
            pixels.SDL_FOURCC("a", "a", "a", 1)
        with pytest.raises(TypeError):
            pixels.SDL_FOURCC("a", "a", 5, 1)
        with pytest.raises(TypeError):
            pixels.SDL_FOURCC("a", 17, 5, 1)
        assert pixels.SDL_FOURCC("0", "0", "0", "0") == 0x30303030
        assert pixels.SDL_FOURCC("1", "1", "1", "1") == 0x31313131
        assert pixels.SDL_FOURCC("1", "2", "3", "4") == 0x34333231
        assert pixels.SDL_FOURCC("4", "3", "2", "1") == 0x31323334

    def test_SDL_DEFINE_PIXELFORMAT(self):
        fmt = pixels.SDL_DEFINE_PIXELFORMAT(pixels.SDL_PIXELTYPE_INDEX1,
                                            pixels.SDL_BITMAPORDER_4321, 0,
                                            1, 0)
        assert fmt == pixels.SDL_PIXELFORMAT_INDEX1LSB

        fmt = pixels.SDL_DEFINE_PIXELFORMAT(pixels.SDL_PIXELTYPE_PACKED16,
                                            pixels.SDL_PACKEDORDER_XRGB,
                                            pixels.SDL_PACKEDLAYOUT_4444,
                                            12, 2)
        assert fmt == pixels.SDL_PIXELFORMAT_RGB444

        fmt = pixels.SDL_DEFINE_PIXELFORMAT(pixels.SDL_PIXELTYPE_PACKED16,
                                            pixels.SDL_PACKEDORDER_ABGR,
                                            pixels.SDL_PACKEDLAYOUT_1555,
                                            16, 2)
        assert fmt == pixels.SDL_PIXELFORMAT_ABGR1555

    def test_SDL_PIXELTYPE(self):
        expected = [
            (pixels.SDL_PIXELFORMAT_INDEX1LSB, pixels.SDL_PIXELTYPE_INDEX1),
            (pixels.SDL_PIXELFORMAT_INDEX1MSB, pixels.SDL_PIXELTYPE_INDEX1),
            (pixels.SDL_PIXELFORMAT_INDEX4LSB, pixels.SDL_PIXELTYPE_INDEX4),
            (pixels.SDL_PIXELFORMAT_ARGB8888, pixels.SDL_PIXELTYPE_PACKED32)
        ]
        for fmt, pxtype in expected:
            assert pixels.SDL_PIXELTYPE(fmt) == pxtype

    def test_SDL_PIXELORDER(self):
        expected = [
            (pixels.SDL_PIXELFORMAT_INDEX1LSB, pixels.SDL_BITMAPORDER_4321),
            (pixels.SDL_PIXELFORMAT_INDEX1MSB, pixels.SDL_BITMAPORDER_1234),
            (pixels.SDL_PIXELFORMAT_INDEX4LSB, pixels.SDL_BITMAPORDER_4321),
            (pixels.SDL_PIXELFORMAT_ARGB8888, pixels.SDL_PACKEDORDER_ARGB)
        ]
        for fmt, pxorder in expected:
            assert pixels.SDL_PIXELORDER(fmt) == pxorder

    def test_SDL_PIXELLAYOUT(self):
        expected = [
            (pixels.SDL_PIXELFORMAT_INDEX1LSB, pixels.SDL_PACKEDLAYOUT_NONE),
            (pixels.SDL_PIXELFORMAT_RGB332, pixels.SDL_PACKEDLAYOUT_332),
            (pixels.SDL_PIXELFORMAT_ARGB8888, pixels.SDL_PACKEDLAYOUT_8888)
        ]
        for fmt, pxlayout in expected:
            assert pixels.SDL_PIXELLAYOUT(fmt) == pxlayout

    def test_SDL_BITSPERPIXEL(self):
        bitspp = pixels.SDL_BITSPERPIXEL
        assert bitspp(pixels.SDL_PIXELFORMAT_INDEX1LSB) == 1
        assert bitspp(pixels.SDL_PIXELFORMAT_INDEX4LSB) == 4
        assert bitspp(pixels.SDL_PIXELFORMAT_RGB332) == 8
        assert bitspp(pixels.SDL_PIXELFORMAT_ARGB8888) == 32
        # TODO: clarify
        # self.assertEqual(bitspp(pixels.SDL_PIXELFORMAT_YUY2), 85)
        # self.assertEqual(bitspp(pixels.SDL_PIXELFORMAT_IYUV), 89)
        # self.assertEqual(bitspp(pixels.SDL_PIXELFORMAT_UYVY), 89)

    def test_SDL_BYTESPERPIXEL(self):
        bytespp = pixels.SDL_BYTESPERPIXEL
        assert bytespp(pixels.SDL_PIXELFORMAT_INDEX1LSB) == 0
        assert bytespp(pixels.SDL_PIXELFORMAT_INDEX4LSB) == 0
        assert bytespp(pixels.SDL_PIXELFORMAT_RGB332) == 1
        assert bytespp(pixels.SDL_PIXELFORMAT_ARGB8888) == 4
        assert bytespp(pixels.SDL_PIXELFORMAT_YUY2) == 2
        assert bytespp(pixels.SDL_PIXELFORMAT_IYUV) == 1
        assert bytespp(pixels.SDL_PIXELFORMAT_UYVY) == 2

    def test_SDL_ISPIXELFORMAT_INDEXED(self):
        isindexed = pixels.SDL_ISPIXELFORMAT_INDEXED
        assert isindexed(pixels.SDL_PIXELFORMAT_INDEX1LSB)
        assert isindexed(pixels.SDL_PIXELFORMAT_INDEX1MSB)
        assert isindexed(pixels.SDL_PIXELFORMAT_INDEX4LSB)
        assert isindexed(pixels.SDL_PIXELFORMAT_INDEX4MSB)
        assert isindexed(pixels.SDL_PIXELFORMAT_INDEX8)
        assert not isindexed(pixels.SDL_PIXELFORMAT_RGB332)
        assert not isindexed(pixels.SDL_PIXELFORMAT_ARGB8888)
        assert not isindexed(pixels.SDL_PIXELFORMAT_YUY2)

    def test_SDL_ISPIXELFORMAT_ALPHA(self):
        isalpha = pixels.SDL_ISPIXELFORMAT_ALPHA
        assert isalpha(pixels.SDL_PIXELFORMAT_ARGB8888)
        assert isalpha(pixels.SDL_PIXELFORMAT_RGBA8888)
        assert isalpha(pixels.SDL_PIXELFORMAT_RGBA4444)
        assert isalpha(pixels.SDL_PIXELFORMAT_ABGR1555)
        assert not isalpha(pixels.SDL_PIXELFORMAT_INDEX1LSB)
        assert not isalpha(pixels.SDL_PIXELFORMAT_INDEX4MSB)
        assert not isalpha(pixels.SDL_PIXELFORMAT_RGB332)
        assert not isalpha(pixels.SDL_PIXELFORMAT_YUY2)
        assert not isalpha(pixels.SDL_PIXELFORMAT_RGBX8888)

    def test_SDL_ISPIXELFORMAT_FOURCC(self):
        isfourcc = pixels.SDL_ISPIXELFORMAT_FOURCC
        assert isfourcc(pixels.SDL_PIXELFORMAT_YV12)
        assert isfourcc(pixels.SDL_PIXELFORMAT_IYUV)
        assert isfourcc(pixels.SDL_PIXELFORMAT_YUY2)
        assert isfourcc(pixels.SDL_PIXELFORMAT_UYVY)
        assert isfourcc(pixels.SDL_PIXELFORMAT_YVYU)
        assert not isfourcc(pixels.SDL_PIXELFORMAT_ARGB8888)
        assert not isfourcc(pixels.SDL_PIXELFORMAT_ARGB4444)
        assert not isfourcc(pixels.SDL_PIXELFORMAT_INDEX8)

    def test_SDL_GetPixelFormatName(self):
        expected = [
            (pixels.SDL_PIXELFORMAT_INDEX1LSB, b"SDL_PIXELFORMAT_INDEX1LSB"),
            (pixels.SDL_PIXELFORMAT_UNKNOWN, b"SDL_PIXELFORMAT_UNKNOWN"),
            (pixels.SDL_PIXELFORMAT_UYVY, b"SDL_PIXELFORMAT_UYVY"),
            (99999, b"SDL_PIXELFORMAT_UNKNOWN")
        ]
        for fmt, name in expected:
            assert pixels.SDL_GetPixelFormatName(fmt) == name

    def test_SDL_MasksToPixelFormatEnum(self):
        if sys.byteorder == "little":
            val = pixels.SDL_MasksToPixelFormatEnum(32,
                                                    0xFF000000,
                                                    0x00FF0000,
                                                    0x0000FF00,
                                                    0x000000FF)
        else:
            val = pixels.SDL_MasksToPixelFormatEnum(32,
                                                   0x000000FF,
                                                   0x0000FF00,
                                                   0x00FF0000,
                                                   0xFF000000)
        assert val == pixels.SDL_PIXELFORMAT_RGBA8888
        if sys.byteorder == "little":
            val = pixels.SDL_MasksToPixelFormatEnum(32,
                                                   0xFF000000,
                                                   0x00FF0000,
                                                   0x0000FF00,
                                                   0)
        else:
            val = pixels.SDL_MasksToPixelFormatEnum(32,
                                                   0,
                                                   0x000000FF,
                                                   0x0000FF00,
                                                   0x00FF0000)
        assert val == pixels.SDL_PIXELFORMAT_RGBX8888
        val = pixels.SDL_MasksToPixelFormatEnum(1, 0, 0, 0, 0)
        assert val == pixels.SDL_PIXELFORMAT_INDEX1MSB  # not LSB
        val = pixels.SDL_MasksToPixelFormatEnum(17, 3, 6, 64, 255)
        assert val == pixels.SDL_PIXELFORMAT_UNKNOWN
        val = pixels.SDL_MasksToPixelFormatEnum(0, 0, 0, 0, 0)
        assert val == pixels.SDL_PIXELFORMAT_UNKNOWN

    def test_SDL_PixelFormatEnumToMasks(self):
        bpp = c_int()
        r, g, b, a = Uint32(), Uint32(), Uint32(), Uint32()
        pixels.SDL_PixelFormatEnumToMasks(pixels.SDL_PIXELFORMAT_INDEX1LSB,
                                          byref(bpp), byref(r), byref(g),
                                          byref(b), byref(a))
        assert (bpp.value, r.value, g.value, b.value, a.value) == \
               (1, 0, 0, 0, 0)
        pixels.SDL_PixelFormatEnumToMasks(pixels.SDL_PIXELFORMAT_INDEX1MSB,
                                          byref(bpp), byref(r), byref(g),
                                          byref(b), byref(a))
        assert (bpp.value, r.value, g.value, b.value, a.value) == \
               (1, 0, 0, 0, 0)

        pixels.SDL_PixelFormatEnumToMasks(pixels.SDL_PIXELFORMAT_RGBA8888,
                                          byref(bpp), byref(r), byref(g),
                                          byref(b), byref(a))
        if sys.byteorder == "little":
            assert (bpp.value, r.value, g.value, b.value, a.value) == \
                   (32, 0xFF000000, 0x00FF0000, 0x0000FF00, 0x000000FF)
        else:
            assert (bpp.value, r.value, g.value, b.value, a.value) == \
                   (32, 0x000000FF, 0x0000FF00, 0x00FF0000, 0xFF000000)
        pixels.SDL_PixelFormatEnumToMasks(pixels.SDL_PIXELFORMAT_RGBX8888,
                                          byref(bpp), byref(r),
                                          byref(g), byref(b),
                                          byref(a))
        if sys.byteorder == "little":
            assert (bpp.value, r.value, g.value, b.value, a.value) == \
                   (32, 0xFF000000, 0x00FF0000, 0x0000FF00, 0)
        else:
            assert (bpp.value, r.value, g.value, b.value, a.value) == \
                   (32, 0, 0x0000FF00, 0x00FF0000, 0xFF000000)
        # self.assertRaises(sdl.SDLError, pixels.SDL_PixelFormatEnumToMasks,
        #                   99999)

        pixels.SDL_PixelFormatEnumToMasks(0, byref(bpp), byref(r), byref(g),
                                          byref(b), byref(a))
        assert (bpp.value, r.value, g.value, b.value, a.value) == \
               (0, 0, 0, 0, 0)
        pixels.SDL_PixelFormatEnumToMasks(pixels.SDL_PIXELFORMAT_UNKNOWN,
                                          byref(bpp), byref(r), byref(g),
                                          byref(b), byref(a))
        assert (bpp.value, r.value, g.value, b.value, a.value) == \
               (0, 0, 0, 0, 0)

    def test_SDL_AllocFreeFormat(self):
        pformat = pixels.SDL_AllocFormat(pixels.SDL_PIXELFORMAT_RGBA8888)
        assert isinstance(pformat.contents, pixels.SDL_PixelFormat)
        assert pformat.contents.format == pixels.SDL_PIXELFORMAT_RGBA8888
        assert pformat.contents.BitsPerPixel == 32
        assert pformat.contents.BytesPerPixel == 4
        pixels.SDL_FreeFormat(pformat)

        pformat = pixels.SDL_AllocFormat(pixels.SDL_PIXELFORMAT_INDEX1LSB)
        assert isinstance(pformat.contents, pixels.SDL_PixelFormat)
        assert pformat.contents.format == pixels.SDL_PIXELFORMAT_INDEX1LSB
        assert pformat.contents.BitsPerPixel == 1
        assert pformat.contents.BytesPerPixel == 1
        pixels.SDL_FreeFormat(pformat)

        pformat = pixels.SDL_AllocFormat(pixels.SDL_PIXELFORMAT_INDEX4MSB)
        assert isinstance(pformat.contents, pixels.SDL_PixelFormat)
        assert pformat.contents.format == pixels.SDL_PIXELFORMAT_INDEX4MSB
        assert pformat.contents.BitsPerPixel == 4
        assert pformat.contents.BytesPerPixel == 1
        pixels.SDL_FreeFormat(pformat)

        # self.assertRaises(sdl.SDLError, pixels.SDL_AllocFormat,
        #                   pixels.SDL_PIXELFORMAT_UYVY)
        # self.assertRaises(sdl.SDLError, pixels.SDL_AllocFormat,
        #                   pixels.SDL_PIXELFORMAT_YUY2)

    def test_SDL_AllocFreePalette(self):
        with pytest.raises((ArgumentError, TypeError)):
            pixels.SDL_AllocPalette(None)
        with pytest.raises((ArgumentError, TypeError)):
            pixels.SDL_AllocPalette("Test")
        # self.assertRaises(ValueError, pixels.SDL_AllocPalette, -5)

        palette = pixels.SDL_AllocPalette(10)
        assert isinstance(palette.contents, pixels.SDL_Palette)
        assert palette.contents.ncolors == 10
        colors = palette.contents.colors
        for x in range(palette.contents.ncolors):
            assert isinstance(colors[x], SDL_Color)
        colors[3].r = 70
        assert colors[3].r == 70
        color = colors[4]
        assert colors[4].g == 255
        assert color.g == 255
        color.g = 33
        assert color.g == 33
        assert colors[4].g == 33
        pixels.SDL_FreePalette(palette)

    def test_SDL_CalculateGammaRamp(self):
        # TODO: more tests
        with pytest.raises(TypeError):
            pixels.SDL_CalculateGammaRamp(None)
        with pytest.raises(TypeError):
            pixels.SDL_CalculateGammaRamp("Test")
        with pytest.raises(TypeError):
            pixels.SDL_CalculateGammaRamp(7)
        with pytest.raises(TypeError):
            pixels.SDL_CalculateGammaRamp(-0.00002)
        vals = (Uint16 * 256)()
        pixels.SDL_CalculateGammaRamp(0, cast(vals, POINTER(Uint16)))
        assert len(vals) == 256
        for x in vals:
            assert x == 0
        vals = (Uint16 * 256)()
        pixels.SDL_CalculateGammaRamp(1, cast(vals, POINTER(Uint16)))
        assert len(vals) == 256
        p = 0
        for x in vals:
            assert x == p
            p += 257
        vals = (Uint16 * 256)()
        pixels.SDL_CalculateGammaRamp(0.5, cast(vals, POINTER(Uint16)))
        assert len(vals) == 256
        p, step = 0, 1
        for x in vals:
            if p == 33124:
                # dubios rounding correction - is this really correct?
                p = 33123
            assert x == p
            p = x + step
            step += 2

    def test_SDL_GetRGB(self):
        # TODO: invalid parameters
        pformat = pixels.SDL_AllocFormat(pixels.SDL_PIXELFORMAT_RGBA8888)
        assert isinstance(pformat.contents, pixels.SDL_PixelFormat)
        r, g, b = Uint8(), Uint8(), Uint8()
        pixels.SDL_GetRGB(0xFFAA8811, pformat, byref(r), byref(g), byref(b))
        assert (r.value, g.value, b.value) == (0xFF, 0xAA, 0x88)
        pixels.SDL_GetRGB(0x00000000, pformat, byref(r), byref(g), byref(b))
        assert (r.value, g.value, b.value) == (0x00, 0x00, 0x00)
        pixels.SDL_GetRGB(0xFFFFFFFF, pformat, byref(r), byref(g), byref(b))
        assert (r.value, g.value, b.value) == (0xFF, 0xFF, 0xFF)
        pixels.SDL_GetRGB(0x11223344, pformat, byref(r), byref(g), byref(b))
        assert (r.value, g.value, b.value) == (0x11, 0x22, 0x33)
        pixels.SDL_FreeFormat(pformat)
        fmts = (pixels.SDL_PIXELFORMAT_INDEX1LSB,
                pixels.SDL_PIXELFORMAT_INDEX1MSB)
        for fmt in fmts:
            pformat = pixels.SDL_AllocFormat(fmt)
            assert isinstance(pformat.contents, pixels.SDL_PixelFormat)
            pixels.SDL_GetRGB(0x11223344, pformat, byref(r), byref(g),
                              byref(b))
            assert (r.value, g.value, b.value) == (0xFF, 0xFF, 0xFF)
            pixels.SDL_GetRGB(0x00000000, pformat, byref(r), byref(g),
                              byref(b))
            # TODO: Seems to be always (0xFF, 0xFF, 0xFF)???
            # self.assertEqual(rgb,(0x00, 0x00, 0x00))
            pixels.SDL_FreeFormat(pformat)
        fmts = (pixels.SDL_PIXELFORMAT_INDEX4LSB,
                pixels.SDL_PIXELFORMAT_INDEX4MSB)
        for fmt in fmts:
            pformat = pixels.SDL_AllocFormat(fmt)
            assert isinstance(pformat.contents, pixels.SDL_PixelFormat)
            # TODO
            pixels.SDL_FreeFormat(pformat)

    def test_SDL_GetRGBA(self):
        # TODO: invalid parameters
        pformat = pixels.SDL_AllocFormat(pixels.SDL_PIXELFORMAT_RGBA8888)
        assert isinstance(pformat.contents, pixels.SDL_PixelFormat)
        r, g, b, a = Uint8(), Uint8(), Uint8(), Uint8()
        pixels.SDL_GetRGBA(0xFFAA8811, pformat, byref(r), byref(g), byref(b),
                           byref(a))
        assert (r.value, g.value, b.value, a.value) == (0xFF, 0xAA, 0x88, 0x11)
        pixels.SDL_GetRGBA(0x00000000, pformat, byref(r), byref(g), byref(b),
                           byref(a))
        assert (r.value, g.value, b.value, a.value) == (0x00, 0x00, 0x00, 0x00)
        pixels.SDL_GetRGBA(0xFFFFFFFF, pformat, byref(r), byref(g), byref(b),
                           byref(a))
        assert (r.value, g.value, b.value, a.value) == (0xFF, 0xFF, 0xFF, 0xFF)
        pixels.SDL_GetRGBA(0x11223344, pformat, byref(r), byref(g), byref(b),
                           byref(a))
        assert (r.value, g.value, b.value, a.value) == (0x11, 0x22, 0x33, 0x44)
        pixels.SDL_FreeFormat(pformat)
        fmts = (pixels.SDL_PIXELFORMAT_INDEX1LSB,
                pixels.SDL_PIXELFORMAT_INDEX1MSB)
        for fmt in fmts:
            pformat = pixels.SDL_AllocFormat(fmt)
            assert isinstance(pformat.contents, pixels.SDL_PixelFormat)
            pixels.SDL_GetRGBA(0x11223344, pformat, byref(r), byref(g),
                               byref(b), byref(a))
            assert (r.value, g.value, b.value, a.value) == \
                   (0xFF, 0xFF, 0xFF, 0xFF)
            pixels.SDL_GetRGBA(0x00000000, pformat, byref(r), byref(g),
                               byref(b), byref(a))
            # TODO: Seems to be always(0xFF, 0xFF, 0xFF) ???
            # self.assertEqual(rgb,(0x00, 0x00, 0x00))
            pixels.SDL_FreeFormat(pformat)
        fmts = (pixels.SDL_PIXELFORMAT_INDEX4LSB,
                pixels.SDL_PIXELFORMAT_INDEX4MSB)
        for fmt in fmts:
            pformat = pixels.SDL_AllocFormat(fmt)
            assert isinstance(pformat.contents, pixels.SDL_PixelFormat)
            # TODO
            pixels.SDL_FreeFormat(pformat)

    def test_SDL_MapRGB(self):
        pformat = pixels.SDL_AllocFormat(pixels.SDL_PIXELFORMAT_RGBA8888)
        assert isinstance(pformat.contents, pixels.SDL_PixelFormat)
        val = pixels.SDL_MapRGB(pformat, 0xFF, 0xAA, 0x88)
        assert val == 0xFFAA88FF
        pixels.SDL_FreeFormat(pformat)

        pformat = pixels.SDL_AllocFormat(pixels.SDL_PIXELFORMAT_UNKNOWN)
        assert isinstance(pformat.contents, pixels.SDL_PixelFormat)
        assert pformat.contents.format == pixels.SDL_PIXELFORMAT_UNKNOWN
        val = pixels.SDL_MapRGB(pformat, 0xFF, 0xAA, 0x88)
        assert val == 0x0
        pixels.SDL_FreeFormat(pformat)

    def test_SDL_MapRGBA(self):
        pformat = pixels.SDL_AllocFormat(pixels.SDL_PIXELFORMAT_RGBA8888)
        assert isinstance(pformat.contents, pixels.SDL_PixelFormat)
        val = pixels.SDL_MapRGBA(pformat, 0xFF, 0xAA, 0x88, 0x11)
        assert val == 0xFFAA8811
        pixels.SDL_FreeFormat(pformat)

        pformat = pixels.SDL_AllocFormat(pixels.SDL_PIXELFORMAT_UNKNOWN)
        assert isinstance(pformat.contents, pixels.SDL_PixelFormat)
        assert pformat.contents.format == pixels.SDL_PIXELFORMAT_UNKNOWN
        val = pixels.SDL_MapRGBA(pformat, 0xFF, 0xAA, 0x88, 0x11)
        assert val == 0x0
        pixels.SDL_FreeFormat(pformat)

    def test_SDL_SetPaletteColors(self):
        colors = []
        for v in range(20):
            colors.append(SDL_Color(v, v + 10, v + 20))

        palette = pixels.SDL_AllocPalette(10)
        assert isinstance(palette.contents, pixels.SDL_Palette)
        assert palette.contents.ncolors == 10
        colors = palette.contents.colors
        for index in range(10):
            rgb = colors[index]
            assert (rgb.r, rgb.g, rgb.b) == (255, 255, 255)
        pixels.SDL_SetPaletteColors(palette, colors, 0, 10)
        for index in range(10):
            rgb = palette.contents.colors[index]
            assert rgb == colors[index]
        pixels.SDL_SetPaletteColors(palette, colors, 5, 1000)
        for index in range(10):
            rgb = palette.contents.colors[index]
            if index < 5:
                assert rgb == colors[index]
            elif index > 5:
                assert rgb == colors[index - 5]

        pixels.SDL_FreePalette(palette)

    def test_SDL_SetPixelFormatPalette(self):
        palette = pixels.SDL_AllocPalette(10)
        assert isinstance(palette.contents, pixels.SDL_Palette)
        pformat = pixels.SDL_AllocFormat(pixels.SDL_PIXELFORMAT_RGBA8888)
        assert isinstance(pformat.contents, pixels.SDL_PixelFormat)
        pixels.SDL_SetPixelFormatPalette(pformat, palette)
        # TODO: improve tests
        pixels.SDL_FreeFormat(pformat)
        pixels.SDL_FreePalette(palette)

    def test_SDL_PixelFormat(self):
        # test_alloc_SDL_FreeFormat() contains the real tests
        pformat = pixels.SDL_PixelFormat()
        assert isinstance(pformat, pixels.SDL_PixelFormat)

    def test_SDL_Palette(self):
        # test_alloc_SDL_FreePalette() contains the real tests
        palette = pixels.SDL_Palette()
        assert isinstance(palette, pixels.SDL_Palette)

    def test_SDL_Color(self):
        c1 = SDL_Color()
        assert (c1.r, c1.g, c1.b, c1.a) == (0xFF, 0xFF, 0xFF, 0xFF)

        c1 = SDL_Color()
        c2 = SDL_Color()
        c3 = SDL_Color(0, 127, 255, 33)
        assert c1 == c2
        assert c1 != c3

    def test_SDL_Color__repr__(self):
        c1 = SDL_Color()
        assert "SDL_Color(r=255, g=255, b=255, a=255)" == repr(c1)
        c2 = eval(repr(c1))
        assert c2 == c1
        c3 = eval(repr(c2))
        assert c3 == c2

    def test_SDL_Color__copy__(self):
        c = SDL_Color()
        c2 = copy.copy(c)
        assert c == c2

        c = SDL_Color(10, 20, 30)
        c2 = copy.copy(c)
        assert c == c2

    def test_SDL_Color__eq__(self):
        assert SDL_Color(255, 0, 0, 0) == SDL_Color(255, 0, 0, 0)
        assert SDL_Color(0, 255, 0, 0) == SDL_Color(0, 255, 0, 0)
        assert SDL_Color(0, 0, 255, 0) == SDL_Color(0, 0, 255, 0)
        assert SDL_Color(0, 0, 0, 255) == SDL_Color(0, 0, 0, 255)
        assert SDL_Color(0, 0, 0, 0) == SDL_Color(0, 0, 0, 0)

        assert not (SDL_Color(0, 0, 0, 0) == SDL_Color(255, 0, 0, 0))
        assert not (SDL_Color(0, 0, 0, 0) == SDL_Color(0, 255, 0, 0))
        assert not (SDL_Color(0, 0, 0, 0) == SDL_Color(0, 0, 255, 0))
        assert not (SDL_Color(0, 0, 0, 0) == SDL_Color(0, 0, 0, 255))

    def test_SDL_Color__ne__(self):
        assert SDL_Color(0, 0, 0, 0) != SDL_Color(255, 0, 0, 0)
        assert SDL_Color(0, 0, 0, 0) != SDL_Color(0, 255, 0, 0)
        assert SDL_Color(0, 0, 0, 0) != SDL_Color(0, 0, 255, 0)
        assert SDL_Color(0, 0, 0, 0) != SDL_Color(0, 0, 255, 0)
        assert SDL_Color(0, 0, 0, 0) != SDL_Color(0, 0, 0, 255)

        assert not (SDL_Color(255, 0, 0, 0) != SDL_Color(255, 0, 0, 0))
        assert not (SDL_Color(0, 255, 0, 0) != SDL_Color(0, 255, 0, 0))
        assert not (SDL_Color(0, 0, 255, 0) != SDL_Color(0, 0, 255, 0))
        assert not (SDL_Color(0, 0, 0, 255) != SDL_Color(0, 0, 0, 255))

    def test_SDL_Color_r(self):
        c1 = SDL_Color()

        def setr(color, val):
            color.r = val

        for x in range(0, 255):
            c1.r = x
            assert c1.r == x

        # TODO
        # self.assertRaises(ValueError, setr,  c1, -1)
        # self.assertRaises(ValueError, setr,  c1, 256)
        with pytest.raises(TypeError):
            setr(c1, "Test")
        with pytest.raises(TypeError):
            setr(c1, None)

    def test_SDL_Color_g(self):
        c1 = SDL_Color()

        def setg(color, val):
            color.g = val

        for x in range(0, 255):
            c1.g = x
            assert c1.g == x

        # TODO
        # self.assertRaises(ValueError, setg,  c1, -1)
        # self.assertRaises(ValueError, setg,  c1, 256)
        with pytest.raises(TypeError):
            setg(c1, "Test")
        with pytest.raises(TypeError):
            setg(c1, None)

    def test_SDL_Color_b(self):
        c1 = SDL_Color()

        def setb(color, val):
            color.b = val

        for x in range(0, 255):
            c1.b = x
            assert c1.b == x

        # TODO
        # self.assertRaises(ValueError, setb,  c1, -1)
        # self.assertRaises(ValueError, setb,  c1, 256)
        with pytest.raises(TypeError):
            setb(c1, "Test")
        with pytest.raises(TypeError):
            setb(c1, None)

    def test_SDL_Color_a(self):
        c1 = SDL_Color()

        def seta(color, val):
            color.a = val

        for x in range(0, 255):
            c1.a = x
            assert c1.a == x

        # TODO
        # self.assertRaises(ValueError, seta,  c1, -1)
        # self.assertRaises(ValueError, seta,  c1, 256)
        with pytest.raises(TypeError):
            seta(c1, "Test")
        with pytest.raises(TypeError):
            seta(c1, None)



import os
import sys
import array
import pytest
from ctypes import c_int, byref, cast, POINTER, c_void_p
import sdl2
from sdl2.ext import CTypesView
from sdl2 import SDL_Init, SDL_Quit, SDL_INIT_EVERYTHING
from sdl2.stdinc import Uint8, Uint32, SDL_TRUE, SDL_FALSE
from sdl2 import blendmode, pixels, rect, surface, rwops, error

to_ctypes = lambda seq, dtype: (dtype * len(seq))(*seq)

alldepths = (1, 4, 8, 12, 15, 16, 24, 32)
indexdepths = (1, 4, 8)
rgbdepths = (8, 12, 15, 16)
rgbadepths = (16, 24, 32)

rgba_pixelations_16x16 = (
    # 32-bit 16x16 RGBA surface
    ([x << 24 for x in range(16 * 16)], 32, 16,
     (0xFF000000, 0x00FF0000, 0x0000FF00, 0x000000FF),
     pixels.SDL_PIXELFORMAT_RGBA8888),
    # 16-bit 16x16 RGBA surface
    ([x << 8 for x in range(16 * 16)], 16, 16,
     (0xF000, 0x0F00, 0x00F0, 0x000F),
     pixels.SDL_PIXELFORMAT_RGB444),
    )

blitsizes = ((2, 2), (5, 5), (10, 10), (20, 20),
             (2, 4), (5, 3), (8, 12), (27, 9),
             )

blitpositions = (
    rect.SDL_Rect(0, 0),
    rect.SDL_Rect(4, 4),
    rect.SDL_Rect(10, 10),
    rect.SDL_Rect(15, 15),
    rect.SDL_Rect(-2, 1),
    rect.SDL_Rect(3, -4),
    rect.SDL_Rect(0, 3),
    rect.SDL_Rect(4, 0),
    rect.SDL_Rect(12, 6),
    rect.SDL_Rect(13, 22),
    )


# TODO: mostly covers positive tests right now - fix this!
class TestSDLSurface(object):
    __tags__ = ["sdl"]

    @classmethod
    def setup_class(cls):
        cls.testfile = os.path.join(os.path.dirname
                                     (os.path.abspath(__file__)),
                                     "resources", "surfacetest.bmp")
        SDL_Init(SDL_INIT_EVERYTHING)

    @classmethod
    def teardown_class(cls):
        SDL_Quit()

    def test_SDL_Surface(self):
        sf = surface.SDL_Surface()
        assert isinstance(sf, surface.SDL_Surface)
        # TODO: property checks

    def test_SDL_ConvertPixels(self):
        for buf, bpp, pitch, masks, fmt in rgba_pixelations_16x16:
            bytecount = bpp // 8
            arflag = None
            if bpp == 32:
                arflag = "I"
            elif bpp == 16:
                arflag = "H"
            src = CTypesView(array.array(arflag, buf), bytecount)
            srcp = cast(src.to_bytes(), POINTER(Uint8))
            dst = (Uint8 * len(src))()
            dstp = cast(dst, POINTER(Uint8))
            ret = surface.SDL_ConvertPixels(16, 16, fmt, srcp, 16 * bytecount,
                                            fmt, dstp, 16 * bytecount)
            assert ret == 0
            for index, val in enumerate(dst):
                assert val == src.view[index]

    def test_SDL_ConvertSurface(self):
        tenbit = [pixels.SDL_PACKEDLAYOUT_2101010, pixels.SDL_PACKEDLAYOUT_1010102]
        for idx in pixels.ALL_PIXELFORMATS:
            pfmt = pixels.SDL_AllocFormat(idx)
            for fmt in pixels.ALL_PIXELFORMATS:
                idx_name = pixels.SDL_GetPixelFormatName(idx).decode('utf-8') # for debug
                fmt_name = pixels.SDL_GetPixelFormatName(fmt).decode('utf-8') # for debug
                # SDL2 doesn't support converting fancier formats (e.g YUV, 10bit)
                if pixels.SDL_ISPIXELFORMAT_FOURCC(idx) or pixels.SDL_PIXELLAYOUT(idx) in tenbit:
                    continue
                if pixels.SDL_ISPIXELFORMAT_FOURCC(fmt) or pixels.SDL_PIXELLAYOUT(fmt) in tenbit:
                    continue
                # SDL2 doesn't support converting to formats w/ less than 8 bpp
                if pixels.SDL_BITSPERPIXEL(idx) < 8:
                    continue
                # SDL2 doesn't support converting from indexed formats w/ 4 bpp
                if pixels.SDL_PIXELTYPE(fmt) == pixels.SDL_PIXELTYPE_INDEX4:
                    continue
                bpp = c_int()
                rmask, gmask, bmask, amask = Uint32(), Uint32(), Uint32(), Uint32()
                ret = pixels.SDL_PixelFormatEnumToMasks(fmt, byref(bpp),
                                                        byref(rmask), byref(gmask),
                                                        byref(bmask), byref(amask))
                assert ret == SDL_TRUE
                sf = surface.SDL_CreateRGBSurface(0, 10, 20, bpp, rmask, gmask,
                                                  bmask, amask)
                assert isinstance(sf.contents, surface.SDL_Surface)
                csf = surface.SDL_ConvertSurface(sf, pfmt, 0)
                assert csf, error.SDL_GetError()
                assert isinstance(csf.contents, surface.SDL_Surface)
                surface.SDL_FreeSurface(sf)
                surface.SDL_FreeSurface(csf)
            pixels.SDL_FreeFormat(pfmt)

        #######################################################################
        # sf = surface.create_rgb_surface(10, 10, 32, 0, 0, 0)
        # self.assertRaises((AttributeError, TypeError),
        #                   surface.convert_surface, None, None, None)
        # self.assertRaises((AttributeError, TypeError),
        #                   surface.convert_surface, sf, None, None)
        # self.assertRaises((AttributeError, TypeError),
        #                   surface.convert_surface, sf, "Test", 0)
        # self.assertRaises((AttributeError, TypeError),
        #                   surface.convert_surface, sf, 12345, 0)
        # self.assertRaises((AttributeError, TypeError),
        #                   surface.convert_surface, None, pfmt, 0)
        # self.assertRaises((AttributeError, TypeError),
        #                   surface.convert_surface, "Test", pfmt, 0)
        # self.assertRaises((AttributeError, TypeError),
        #                   surface.convert_surface, 12345, pfmt, 0)
        # surface.free_surface(sf)
        #######################################################################

    def test_SDL_ConvertSurfaceFormat(self):
        tenbit = [pixels.SDL_PACKEDLAYOUT_2101010, pixels.SDL_PACKEDLAYOUT_1010102]
        for pfmt in pixels.ALL_PIXELFORMATS:
            for fmt in pixels.ALL_PIXELFORMATS:
                # SDL2 doesn't support converting fancier formats (e.g YUV, 10bit)
                if pixels.SDL_ISPIXELFORMAT_FOURCC(pfmt) or pixels.SDL_PIXELLAYOUT(pfmt) in tenbit:
                    continue
                if pixels.SDL_ISPIXELFORMAT_FOURCC(fmt) or pixels.SDL_PIXELLAYOUT(fmt) in tenbit:
                    continue
                # SDL2 doesn't support converting to formats w/ less than 8 bpp
                if pixels.SDL_BITSPERPIXEL(pfmt) < 8:
                    continue
                # SDL2 doesn't support converting from indexed formats w/ 4 bpp
                if pixels.SDL_PIXELTYPE(fmt) == pixels.SDL_PIXELTYPE_INDEX4:
                    continue
                bpp = c_int()
                rmask, gmask, bmask, amask = Uint32(), Uint32(), Uint32(), Uint32()
                ret = pixels.SDL_PixelFormatEnumToMasks(fmt, byref(bpp),
                                                        byref(rmask), byref(gmask),
                                                        byref(bmask), byref(amask))
                assert ret == SDL_TRUE
                sf = surface.SDL_CreateRGBSurface(0, 10, 20, bpp, rmask, gmask,
                                                  bmask, amask)
                assert isinstance(sf.contents, surface.SDL_Surface)
                csf = surface.SDL_ConvertSurfaceFormat(sf, pfmt, 0)
                assert csf, error.SDL_GetError()
                assert isinstance(csf.contents, surface.SDL_Surface)
                surface.SDL_FreeSurface(sf)
                surface.SDL_FreeSurface(csf)

    def test_SDL_CreateRGBSurface(self):
        for w in range(1, 100, 5):
            for h in range(1, 100, 5):
                for bpp in alldepths:
                    sf = surface.SDL_CreateRGBSurface(0, w, h, bpp, 0, 0, 0, 0)
                    assert isinstance(sf.contents, surface.SDL_Surface)
                    surface.SDL_FreeSurface(sf)

        bpp = c_int()
        rmask, gmask, bmask, amask = Uint32(), Uint32(), Uint32(), Uint32()
        for fmt in pixels.ALL_PIXELFORMATS:
            if pixels.SDL_ISPIXELFORMAT_FOURCC(fmt):
                continue
            for w in range(1, 100, 5):
                for h in range(1, 100, 5):
                    ret = pixels.SDL_PixelFormatEnumToMasks(fmt, byref(bpp),
                                                            byref(rmask),
                                                            byref(gmask),
                                                            byref(bmask),
                                                            byref(amask))
                    assert ret == SDL_TRUE
                    sf = surface.SDL_CreateRGBSurface(0, 10, 20, bpp, rmask,
                                                      gmask, bmask, amask)
                    assert isinstance(sf.contents, surface.SDL_Surface)
                    surface.SDL_FreeSurface(sf)

        # Broken ones
        # sizes = [(0, 0), (65536, 65536)]
        # for (w, h) in sizes:
        #    for bpp in alldepths:
        #        print w, h, bpp
        #        sf = surface.create_rgb_surface(w, h, bpp)
        #        self.assertIsInstance(sf, surface.SDL_Surface)
        #        surface.free_surface(sf)

        # TODO: raises a SDL Out of memory error, but 65536 works?!
        #
        # sf = surface.create_rgb_surface(65535, 65535, 32)
        # self.assertIsInstance(sf, surface.SDL_Surface)

        #self.assertRaises(sdl.SDLError, surface.create_rgb_surface, 1, 1, 66)
        #self.assertRaises(sdl.SDLError, surface.create_rgb_surface, 1, 1, 0)
        #self.assertRaises(sdl.SDLError, surface.create_rgb_surface, 1, 1, 8,
        #                  0xf0, 0x0f, 0x01, 0x02)
        #self.assertRaises(sdl.SDLError, surface.create_rgb_surface, 1, 1, 16,
        #                  0xf0, 0x0f, 0x01, 0x02)
        #self.assertRaises(sdl.SDLError, surface.create_rgb_surface, 1, 1, 24,
        #                  0xf0, 0x0f, 0x01, 0x02)
        #self.assertRaises(sdl.SDLError, surface.create_rgb_surface, 1, 1, 32,
        #                  0xf0, 0x0f, 0x01, 0x02)

    def test_SDL_CreateRGBSurfaceWithFormat(self):
        for w in range(1, 100, 5):
            for h in range(1, 100, 5):
                for bpp in alldepths:
                    for fmt in pixels.ALL_PIXELFORMATS:
                        if pixels.SDL_ISPIXELFORMAT_FOURCC(fmt):
                            continue
                        sf = surface.SDL_CreateRGBSurfaceWithFormat(0, w, h,
                                                                    bpp, fmt)
                        assert isinstance(sf.contents, surface.SDL_Surface)
                        surface.SDL_FreeSurface(sf)

    def test_SDL_CreateRGBSurfaceWithFormatFrom(self):
        for buf, bpp, pitch, masks, fmt in rgba_pixelations_16x16:
            if bpp == 32:
                arflag = "I"
            elif bpp == 16:
                arflag = "H"
            bytebuf = CTypesView(array.array(arflag, buf))
            bufptr = cast(bytebuf.to_bytes(), POINTER(Uint8))
            sf = surface.SDL_CreateRGBSurfaceWithFormatFrom(bufptr, 16, 16,
                    bpp, pitch, fmt)
            assert isinstance(sf.contents, surface.SDL_Surface)
            surface.SDL_FreeSurface(sf)

    def test_SDL_CreateRGBSurfaceFrom(self):
        for buf, bpp, pitch, masks, fmt in rgba_pixelations_16x16:
            if bpp == 32:
                arflag = "I"
            elif bpp == 16:
                arflag = "H"
            bytebuf = CTypesView(array.array(arflag, buf))
            bufptr = cast(bytebuf.to_bytes(), POINTER(Uint8))
            sf = surface.SDL_CreateRGBSurfaceFrom(bufptr, 16, 16, bpp,
                                                  pitch, masks[0], masks[1],
                                                  masks[2], masks[3])
            assert isinstance(sf.contents, surface.SDL_Surface)
            surface.SDL_FreeSurface(sf)

    def test_SDL_FillRect(self):
        rectlist = (rect.SDL_Rect(0, 0, 0, 0),
                    rect.SDL_Rect(0, 0, 10, 10),
                    rect.SDL_Rect(0, 0, -10, 10),
                    rect.SDL_Rect(0, 0, -10, -10),
                    rect.SDL_Rect(-10, -10, 10, 10),
                    rect.SDL_Rect(10, -10, 10, 10),
                    rect.SDL_Rect(10, 10, 10, 10),
                    )

        bpp = c_int()
        rmask, gmask, bmask, amask = Uint32(), Uint32(), Uint32(), Uint32()
        for fmt in pixels.ALL_PIXELFORMATS:
            if pixels.SDL_ISPIXELFORMAT_FOURCC(fmt):
                continue
            if pixels.SDL_BITSPERPIXEL(fmt) < 8:
                continue  # Skip < 8bpp, SDL_FillRect does not work on those

            ret = pixels.SDL_PixelFormatEnumToMasks(fmt, byref(bpp),
                                                    byref(rmask), byref(gmask),
                                                    byref(bmask), byref(amask))
            assert ret == SDL_TRUE
            for w in range(1, 100, 5):
                for h in range(1, 100, 5):
                    sf = surface.SDL_CreateRGBSurface(0, w, h, bpp, rmask,
                                                      gmask, bmask, amask)
                    for r in rectlist:
                        # TODO: check for changed pixels
                        ret = surface.SDL_FillRect(sf, r, 0xff00ff00)
                        assert ret == 0
                    surface.SDL_FreeSurface(sf)

    def test_SDL_FillRects(self):
        rectlist = to_ctypes([rect.SDL_Rect(0, 0, 0, 0),
                              rect.SDL_Rect(0, 0, 10, 10),
                              rect.SDL_Rect(0, 0, -10, 10),
                              rect.SDL_Rect(0, 0, -10, -10),
                              rect.SDL_Rect(-10, -10, 10, 10),
                              rect.SDL_Rect(10, -10, 10, 10),
                              rect.SDL_Rect(10, 10, 10, 10)], rect.SDL_Rect)

        bpp = c_int()
        rmask, gmask, bmask, amask = Uint32(), Uint32(), Uint32(), Uint32()
        for fmt in pixels.ALL_PIXELFORMATS:
            if pixels.SDL_ISPIXELFORMAT_FOURCC(fmt):
                continue
            if pixels.SDL_BITSPERPIXEL(fmt) < 8:
                continue  # Skip < 8bpp, SDL_FillRect does not work on those
            ret = pixels.SDL_PixelFormatEnumToMasks(fmt, byref(bpp),
                                                    byref(rmask), byref(gmask),
                                                    byref(bmask), byref(amask))
            assert ret == SDL_TRUE
            for w in range(1, 100, 5):
                for h in range(1, 100, 5):
                    sf = surface.SDL_CreateRGBSurface(0, w, h, bpp, rmask,
                                                      gmask, bmask, amask)
                    # TODO: check for changed pixels
                    surface.SDL_FillRects(sf, rectlist, 7, 0xff00ff00)
                    surface.SDL_FreeSurface(sf)

    def test_SDL_FreeSurface(self):
#        self.assertRaises((AttributeError, TypeError),
#                          surface.SDL_FreeSurface, None)
#        self.assertRaises((AttributeError, TypeError),
#                          surface.free_surface, "Test")
#        self.assertRaises((AttributeError, TypeError),
#                          surface.free_surface, 5)

        formats = (pixels.SDL_PIXELFORMAT_INDEX1LSB,
                   pixels.SDL_PIXELFORMAT_RGB332,
                   pixels.SDL_PIXELFORMAT_RGBA4444,
                   pixels.SDL_PIXELFORMAT_BGR888,
                   pixels.SDL_PIXELFORMAT_ARGB8888,
                   pixels.SDL_PIXELFORMAT_ARGB2101010
                   )
        bpp = c_int()
        rmask, gmask, bmask, amask = Uint32(), Uint32(), Uint32(), Uint32()
        for fmt in formats:
            ret = pixels.SDL_PixelFormatEnumToMasks(fmt, byref(bpp),
                                                    byref(rmask), byref(gmask),
                                                    byref(bmask), byref(amask))
            assert ret == SDL_TRUE
            sf = surface.SDL_CreateRGBSurface(0, 5, 5, bpp, rmask, gmask,
                                              bmask, amask)
            surface.SDL_FreeSurface(sf)

    def test_SDL_GetSetClipRect(self):
        rectlist = ((rect.SDL_Rect(0, 0, 0, 0), SDL_FALSE, True),
                    (rect.SDL_Rect(2, 2, 0, 0), SDL_FALSE, False),
                    (rect.SDL_Rect(2, 2, 5, 1), SDL_TRUE, True),
                    (rect.SDL_Rect(6, 5, 10, 3), SDL_TRUE, False),
                    (rect.SDL_Rect(0, 0, 10, 10), SDL_TRUE, True),
                    (rect.SDL_Rect(0, 0, -10, 10), SDL_FALSE, False),
                    (rect.SDL_Rect(0, 0, -10, -10), SDL_FALSE, False),
                    (rect.SDL_Rect(-10, -10, 10, 10), SDL_FALSE, False),
                    (rect.SDL_Rect(10, -10, 10, 10), SDL_FALSE, False),
                    (rect.SDL_Rect(10, 10, 10, 10), SDL_TRUE, False)
                    )

        clip = rect.SDL_Rect()
        sf = surface.SDL_CreateRGBSurface(0, 15, 15, 32, 0, 0, 0, 0)
        assert isinstance(sf.contents, surface.SDL_Surface)
        surface.SDL_GetClipRect(sf, byref(clip))
        assert clip == rect.SDL_Rect(0, 0, 15, 15)

        for r, clipsetval, cmpval in rectlist:
            retval = surface.SDL_SetClipRect(sf, r)
            surface.SDL_GetClipRect(sf, byref(clip))
            err = "Could not set clip rect %s" % r
            assert retval == clipsetval, "retval: " + err
            assert (clip == r) == cmpval, "clip: " + err
        surface.SDL_FreeSurface(sf)

    @pytest.mark.skipif(sdl2.dll.version < 2009, reason="not available")
    def test_SDL_HasColorKey(self):
        sf = surface.SDL_CreateRGBSurface(0, 10, 10, 32, 0, 0, 0, 0)
        assert surface.SDL_HasColorKey(sf) == SDL_FALSE
        key = pixels.SDL_MapRGB(sf.contents.format, 255, 255, 255)
        surface.SDL_SetColorKey(sf, 1, key)
        assert surface.SDL_HasColorKey(sf) == SDL_TRUE
        surface.SDL_FreeSurface(sf)

    def test_SDL_GetSetColorKey(self):
        colorkeys = ((0, 0, 0),
                     (32, 64, 128),
                     (10, 20, 30),
                     (1, 2, 4),
                     (255, 255, 255),
                     (128, 128, 128),
                     (127, 127, 127),
                     )
        bpp = c_int()
        rmask, gmask, bmask, amask = Uint32(), Uint32(), Uint32(), Uint32()
        for fmt in pixels.ALL_PIXELFORMATS:
            if pixels.SDL_ISPIXELFORMAT_FOURCC(fmt):
                continue
            pformat = pixels.SDL_AllocFormat(fmt)
            assert isinstance(pformat.contents, pixels.SDL_PixelFormat)
            ret = pixels.SDL_PixelFormatEnumToMasks(fmt, byref(bpp),
                                                    byref(rmask), byref(gmask),
                                                    byref(bmask), byref(amask))
            assert ret == SDL_TRUE
            sf = surface.SDL_CreateRGBSurface(0, 10, 10, bpp, rmask, gmask,
                                              bmask, amask)
            for r, g, b in colorkeys:
                key = pixels.SDL_MapRGB(pformat, r, g, b)
                surface.SDL_SetColorKey(sf, 1, key)
                skey = Uint32()
                ret = surface.SDL_GetColorKey(sf, byref(skey))
                errmsg = "Could not set color key (%d, %d, %d)" % (r, g, b)
                assert ret == 0
                assert skey.value == key, errmsg
                    
            pixels.SDL_FreeFormat(pformat)
            surface.SDL_FreeSurface(sf)

    def test_SDL_GetSetSurfaceAlphaMod(self):
        bpp = c_int()
        rmask, gmask, bmask, amask = Uint32(), Uint32(), Uint32(), Uint32()
        for fmt in pixels.ALL_PIXELFORMATS:
            if pixels.SDL_ISPIXELFORMAT_FOURCC(fmt):
                continue
            ret = pixels.SDL_PixelFormatEnumToMasks(fmt, byref(bpp),
                                                    byref(rmask), byref(gmask),
                                                    byref(bmask), byref(amask))
            assert ret == SDL_TRUE
            sf = surface.SDL_CreateRGBSurface(0, 10, 10, bpp, rmask, gmask,
                                              bmask, amask)
            salpha = Uint8()
            ret = surface.SDL_GetSurfaceAlphaMod(sf, byref(salpha))
            assert ret == 0
            assert salpha.value == 255
            for alpha in range(0, 255):
                ret = surface.SDL_SetSurfaceAlphaMod(sf, alpha)
                assert ret == 0
                ret = surface.SDL_GetSurfaceAlphaMod(sf, byref(salpha))
                assert ret == 0
                assert salpha.value == alpha
            surface.SDL_FreeSurface(sf)

    def test_SDL_GetSetSurfaceBlendMode(self):
        modes = (blendmode.SDL_BLENDMODE_NONE,
                 blendmode.SDL_BLENDMODE_BLEND,
                 blendmode.SDL_BLENDMODE_ADD,
                 blendmode.SDL_BLENDMODE_MOD
                 )
        bpp = c_int()
        rmask, gmask, bmask, amask = Uint32(), Uint32(), Uint32(), Uint32()
        for fmt in pixels.ALL_PIXELFORMATS:
            if pixels.SDL_ISPIXELFORMAT_FOURCC(fmt):
                continue
            ret = pixels.SDL_PixelFormatEnumToMasks(fmt, byref(bpp),
                                                    byref(rmask), byref(gmask),
                                                    byref(bmask), byref(amask))
            assert ret == SDL_TRUE
            sf = surface.SDL_CreateRGBSurface(0, 10, 10, bpp, rmask, gmask,
                                              bmask, amask)
            for mode in modes:
                smode = blendmode.SDL_BlendMode()
                ret = surface.SDL_SetSurfaceBlendMode(sf, mode)
                assert ret == 0
                surface.SDL_GetSurfaceBlendMode(sf, byref(smode))
                assert smode.value == mode
            surface.SDL_FreeSurface(sf)

    def test_SDL_GetSetSurfaceColorMod(self):
        colormods = ((0, 0, 0),
                     (32, 64, 128),
                     (10, 20, 30),
                     (1, 2, 4),
                     (255, 255, 255),
                     (128, 128, 128),
                     (127, 127, 127),
                     )
        bpp = c_int()
        rmask, gmask, bmask, amask = Uint32(), Uint32(), Uint32(), Uint32()
        for fmt in pixels.ALL_PIXELFORMATS:
            if pixels.SDL_ISPIXELFORMAT_FOURCC(fmt):
                continue
            ret = pixels.SDL_PixelFormatEnumToMasks(fmt, byref(bpp),
                                                    byref(rmask), byref(gmask),
                                                    byref(bmask), byref(amask))
            assert ret == SDL_TRUE
            sf = surface.SDL_CreateRGBSurface(0, 10, 10, bpp, rmask, gmask,
                                              bmask, amask)
            for r, g, b in colormods:
                sr, sg, sb = Uint8(), Uint8(), Uint8()
                surface.SDL_SetSurfaceColorMod(sf, r, g, b)
                ret = surface.SDL_GetSurfaceColorMod(sf, byref(sr), byref(sg),
                                                     byref(sb))
                assert ret == 0
                assert (sr.value, sg.value, sb.value) == (r, g, b)
            surface.SDL_FreeSurface(sf)

    def test_SDL_LockUnlockMUSTLOCKSurface(self):
        bpp = c_int()
        rmask, gmask, bmask, amask = Uint32(), Uint32(), Uint32(), Uint32()
        for fmt in pixels.ALL_PIXELFORMATS:
            if pixels.SDL_ISPIXELFORMAT_FOURCC(fmt):
                continue
            ret = pixels.SDL_PixelFormatEnumToMasks(fmt, byref(bpp),
                                                    byref(rmask), byref(gmask),
                                                    byref(bmask), byref(amask))
            assert ret == SDL_TRUE
            sf = surface.SDL_CreateRGBSurface(0, 10, 10, bpp, rmask, gmask,
                                              bmask, amask)
            # TODO: locking seems to be only necessary for RLE surfaces...
            if surface.SDL_MUSTLOCK(sf.contents):
                assert not sf.contents.locked
                surface.SDL_LockSurface(sf)
                assert sf.contents.locked
                surface.SDL_LockSurface(sf)
                assert sf.contents.locked
                surface.SDL_LockSurface(sf)
                assert sf.contents.locked
                surface.SDL_LockSurface(sf)
                assert sf.contents.locked
                surface.SDL_LockSurface(sf)
                assert sf.contents.locked
                surface.SDL_UnlockSurface(sf)
                assert not sf.contents.locked
            surface.SDL_FreeSurface(sf)

    def test_SDL_LowerBlit(self):
        bpp = 32
        w, h = 10, 10
        # no alpha to prevent blending
        masks = (0xFF000000, 0x00FF0000, 0x0000FF00, 0x00000000)
        dest = surface.SDL_CreateRGBSurface(0, w, h, bpp, masks[0], masks[1],
                                            masks[2], masks[3])
        pixelsize = h * dest.contents.pitch
        rowlen = dest.contents.pitch // 4

        sources = []
        for width, height in blitsizes:
            src = surface.SDL_CreateRGBSurface(0, width, height, bpp, masks[0],
                                               masks[1], masks[2], masks[3])
            surface.SDL_FillRect(src, None, 0xFFFFFFFF)  # fill with white
            sources.append(src)

        for src in sources:
            for pos in blitpositions:
                drect = pos.__copy__()
                surface.SDL_FillRect(dest, None, 0x0)  # fill with black
                surface.SDL_LowerBlit(src, src.contents.clip_rect, dest, drect)
                buf = cast(dest.contents.pixels, POINTER(Uint8 * pixelsize))
                pbuf = CTypesView(buf.contents, itemsize=1, objsize=pixelsize)
                iview = pbuf.to_uint32()
                pw = drect.x + drect.w
                ph = drect.y + drect.h
                e = "color mismatch at %d,%d for %s: %d != %d"
                for y in range(dest.contents.h):
                    for x in range(dest.contents.w):
                        col = iview[y * rowlen + x]
                        if y >= drect.y and y < ph and x >= drect.x and x < pw:
                            assert col == 0xFFFFFFFF, \
                                   e % (y, x, pos, col, 0xFFFFFFFF)
                        else:
                            assert col == 0x0, e % (y, x, pos, col, 0x0)

        while len(sources) > 0:
            sf = sources.pop()
            surface.SDL_FreeSurface(sf)
        surface.SDL_FreeSurface(dest)

    @pytest.mark.skip("not implemented")
    def test_SDL_LowerBlitScaled(self):
        pass

    def test_SDL_UpperBlit(self):
        bpp = 32
        w, h = 10, 10
        # no alpha to prevent blending
        masks = (0xFF000000, 0x00FF0000, 0x0000FF00, 0x00000000)
        dest = surface.SDL_CreateRGBSurface(0, w, h, bpp, masks[0], masks[1],
                                            masks[2], masks[3])
        pixelsize = h * dest.contents.pitch
        rowlen = dest.contents.pitch // 4

        sources = []
        for width, height in blitsizes:
            src = surface.SDL_CreateRGBSurface(0, width, height, bpp, masks[0],
                                               masks[1], masks[2], masks[3])
            surface.SDL_FillRect(src, None, 0xFFFFFFFF)  # fill with white
            sources.append(src)

        for src in sources:
            for pos in blitpositions:
                drect = pos.__copy__()
                surface.SDL_FillRect(dest, None, 0x0)  # fill with black
                surface.SDL_UpperBlit(src, None, dest, drect)
                buf = cast(dest.contents.pixels, POINTER(Uint8 * pixelsize))
                pbuf = CTypesView(buf.contents, itemsize=1, objsize=pixelsize)
                iview = pbuf.to_uint32()
                pw = drect.x + drect.w
                ph = drect.y + drect.h
                e = "color mismatch at %d,%d for %s: %d != %d"
                for y in range(dest.contents.h):
                    for x in range(dest.contents.w):
                        col = iview[y * rowlen + x]
                        if y >= drect.y and y < ph and x >= drect.x and x < pw:
                            assert col == 0xFFFFFFFF, \
                                   e % (x, y, pos, col, 0xFFFFFFFF)
                        else:
                            assert col == 0x0, e % (x, y, pos, col, 0x0)

        while len(sources) > 0:
            sf = sources.pop()
            surface.SDL_FreeSurface(sf)
        surface.SDL_FreeSurface(dest)

    def test_SDL_BlitSurface(self):
        bpp = 32
        w, h = 10, 10
        # no alpha to prevent blending
        masks = (0xFF000000, 0x00FF0000, 0x0000FF00, 0x00000000)
        dest = surface.SDL_CreateRGBSurface(0, w, h, bpp, masks[0], masks[1],
                                            masks[2], masks[3])
        pixelsize = h * dest.contents.pitch
        rowlen = dest.contents.pitch // 4

        sources = []
        for width, height in blitsizes:
            src = surface.SDL_CreateRGBSurface(0, width, height, bpp, masks[0],
                                               masks[1], masks[2], masks[3])
            surface.SDL_FillRect(src, None, 0xFFFFFFFF)  # fill with white
            sources.append(src)

        for src in sources:
            for pos in blitpositions:
                drect = pos.__copy__()
                surface.SDL_FillRect(dest, None, 0x0)  # fill with black
                surface.SDL_BlitSurface(src, None, dest, drect)
                buf = cast(dest.contents.pixels, POINTER(Uint8 * pixelsize))
                pbuf = CTypesView(buf.contents, itemsize=1, objsize=pixelsize)
                iview = pbuf.to_uint32()
                pw = drect.x + drect.w
                ph = drect.y + drect.h
                e = "color mismatch at %d,%d for %s: %d != %d"
                for y in range(dest.contents.h):
                    for x in range(dest.contents.w):
                        col = iview[y * rowlen + x]
                        if y >= drect.y and y < ph and x >= drect.x and x < pw:
                            assert col == 0xFFFFFFFF, \
                                   e % (y, x, pos, col, 0xFFFFFFFF)
                        else:
                            assert col == 0x0, e % (y, x, pos, col, 0x0)

        while len(sources) > 0:
            sf = sources.pop()
            surface.SDL_FreeSurface(sf)
        surface.SDL_FreeSurface(dest)

    @pytest.mark.skip("not implemented")
    def test_SDL_UpperBlitScaled(self):
        pass

    @pytest.mark.skip("not implemented")
    def test_SDL_SoftStretch(self):
        pass

    def test_SDL_SetSurfacePalette(self):
        invpalette = pixels.SDL_AllocPalette(10)
        palette = pixels.SDL_AllocPalette(1 << 16)
        sf = surface.SDL_CreateRGBSurface(0, 10, 10, 16, 0, 0, 0, 0)

        assert not sf.contents.format.contents.palette
        surface.SDL_SetSurfacePalette(sf, palette)
        assert sf.contents.format.contents.palette.contents is not None
        assert sf.contents.format.contents.palette.contents is not None
        surface.SDL_SetSurfacePalette(sf, None)
        assert not sf.contents.format.contents.palette

        surface.SDL_FreeSurface(sf)
        pixels.SDL_FreePalette(invpalette)
        pixels.SDL_FreePalette(palette)

    @pytest.mark.skip("not implemented")
    def test_SDL_SetSurfaceRLE(self):
        pass

    def test_SDL_LoadBMP_RW(self):
        fp = open(self.testfile, "rb")
        imgrw = rwops.rw_from_object(fp)
        imgsurface = surface.SDL_LoadBMP_RW(imgrw, 0)
        assert isinstance(imgsurface.contents, surface.SDL_Surface)
        surface.SDL_FreeSurface(imgsurface)
        imgrw.close(imgrw)

    def test_SDL_LoadBMP(self):
        imgsurface = surface.SDL_LoadBMP(self.testfile.encode("utf-8"))
        assert isinstance(imgsurface.contents, surface.SDL_Surface)
        surface.SDL_FreeSurface(imgsurface)

    @pytest.mark.skip("not implemented")
    def test_SDL_SaveBMP_RW(self):
        pass

    @pytest.mark.skip("not implemented")
    def test_SDL_SaveBMP(self):
        pass

    @pytest.mark.skip("not implemented")
    def test_SDL_GetSetYUVConversionMode(self):
        pass

    @pytest.mark.skip("not implemented")
    def test_SDL_GetYUVConversionModeForResolution(self):
        pass



"""Direct pixel access examples."""
import sys
import sdl2
import sdl2.ext

# Define black and white as global values, so we can access them throughout
# the code.
BLACK = sdl2.ext.Color(0, 0, 0)
WHITE = sdl2.ext.Color(255, 255, 255)


# This function will use a rectangular area and fill each second horizontal
# line with a white color on the passed surface.
def draw_horizontal_stripes(surface, x1, x2, y1, y2):
    # Fill the entire surface with a black color. In contrast to
    # colorpalettes.py we use a Color() value here, just to demonstrate that
    # it really works.
    sdl2.ext.fill(surface, BLACK)

    # Create a 2D view that allows us to directly access each individual pixel
    # of the surface. The PixelView class is quite slow, since it uses an non-
    # optimised read-write access to each individual pixel and offset. It
    # works on every platform, though.
    pixelview = sdl2.ext.PixelView(surface)

    # Loop over the area bounds, considering each fourth line and every column
    # on the 2D view. The PixelView uses a y-x alignment to access pixels.
    # This mkeans that the first accessible dimension of the PixelView denotes
    # the horizontal lines of an image, and the second the vertical lines.
    for y in range(y1, y2, 4):
        for x in range(x1, x2):
            # Change the color of each individual pixel. We can assign any
            # color-like value here, since the assignment method of the
            # PixelView will implicitly check and convert the value to a
            # matching color for its target surface.
            pixelview[y][x] = WHITE

    # Explicitly delete the PixelView. Some surface types need to be locked
    # in order to access their pixels directly. The PixelView will do that
    # implicitly at creation time. Once we are done with all necessary
    # operations, we need to unlock the surface, which will be done
    # automatically at the time the PixelView is garbage-collected.
    del pixelview


# as draw_horizontal_stripes(), but vertical
def draw_vertical_stripes(surface, x1, x2, y1, y2):
    sdl2.ext.fill(surface, BLACK)
    pixelview = sdl2.ext.PixelView(surface)
    for x in range(x1, x2, 4):
        for y in range(y1, y2):
            pixelview[y][x] = WHITE
    del pixelview


def run():
    # You know those from the helloworld.py example.
    # Initialize the video subsystem, create a window and make it visible.
    sdl2.ext.init()
    window = sdl2.ext.Window("Pixel Access", size=(800, 600))
    window.show()

    # As in colorpalettes.py, explicitly acquire the window's surface to
    # draw on.
    windowsurface = window.get_surface()

    # We implement the functionality as it was done in colorpalettes.py and
    # utilise a mapping table to look up the function to be executed, together
    # with the arguments they should receive
    functions = ((draw_horizontal_stripes, (windowsurface, 300, 500, 200, 400)),
                 (draw_vertical_stripes, (windowsurface, 300, 500, 200, 400)),
                 )

    # A storage variable for the function we are currently on, so that we know
    # which function to execute next.
    curindex = 0
    draw_horizontal_stripes(windowsurface, 300, 500, 200, 400)

    # The event loop is nearly the same as we used in colorpalettes.py. If you
    # do not know, what happens here, take a look at colorpalettes.py for a
    # detailled description.
    running = True
    while running:
        events = sdl2.ext.get_events()
        for event in events:
            if event.type == sdl2.SDL_QUIT:
                running = False
                break
            if event.type == sdl2.SDL_MOUSEBUTTONDOWN:
                curindex += 1
                if curindex >= len(functions):
                    curindex = 0
                # In contrast to colorpalettes.py, our mapping table consists
                # of functions and their arguments. Thus, we get the currently
                # requested function and argument tuple and execute the
                # function with the arguments.
                func, args = functions[curindex]
                func(*args)
                break
        window.refresh()
    sdl2.ext.quit()
    return 0


if __name__ == "__main__":
    sys.exit(run())



"""Window routines to manage on-screen windows."""
from ctypes import c_int, byref
from .compat import byteify, stringify
from .common import SDLError
from .. import video

__all__ = ["Window"]


class Window(object):
    """The Window class represents a visible on-screen object with an
    optional border and title text.

    It represents an area on the screen that can be accessed by the
    application for displaying graphics and receive and process user
    input.
    """
    DEFAULTFLAGS = video.SDL_WINDOW_HIDDEN
    DEFAULTPOS = (video.SDL_WINDOWPOS_UNDEFINED,
                  video.SDL_WINDOWPOS_UNDEFINED)

    def __init__(self, title, size, position=None, flags=None):
        """Creates a Window with a specific size and title.

        The position to show the Window at is undefined by default,
        letting the operating system or window manager pick the best
        location. The behaviour can be adjusted through the DEFAULTPOS
        class variable:

            Window.DEFAULTPOS = (10, 10)

        The created Window is hidden by default, which can be overriden
        at the time of creation by providing other SDL window flags
        through the flags parameter.

        The default flags for creating Window instances can be adjusted
        through the DEFAULTFLAGS class variable:

            Window.DEFAULTFLAGS = sdl2.video.SDL_WINDOW_SHOWN
        """
        if position is None:
            position = self.DEFAULTPOS
        if flags is None:
            flags = self.DEFAULTFLAGS

        self._title = title
        self._position = position
        self._flags = flags
        self._size = size

        self.window = None
        self.create()

    def __del__(self):
        """Releases the resources of the Window, implicitly destroying the
        underlying SDL2 window."""
        self.close()

    @property
    def position(self):
        """The current window position as two-value tuple."""
        x, y = c_int(), c_int()
        video.SDL_GetWindowPosition(self.window, byref(x), byref(y))
        return x.value, y.value

    @position.setter
    def position(self, value):
        """The current window position as two-value tuple."""
        if not self.window:
            raise SDLError("The window is not available")
        video.SDL_SetWindowPosition(self.window, value[0], value[1])
        self._position = value[0], value[1]

    @property
    def title(self):
        """The title of the window."""
        return stringify(video.SDL_GetWindowTitle(self.window), "utf-8")

    @title.setter
    def title(self, value):
        """The title of the window."""
        video.SDL_SetWindowTitle(self.window, byteify(value, "utf-8"))
        self._title = value

    @property
    def size(self):
        """The size of the window."""
        w, h = c_int(), c_int()
        video.SDL_GetWindowSize(self.window, byref(w), byref(h))
        return w.value, h.value

    @size.setter
    def size(self, value):
        """The size of the window."""
        video.SDL_SetWindowSize(self.window, value[0], value[1])
        self._size = value[0], value[1]

    def create(self):
        """Creates the window.

        If the window already exists, this method will not do anything.
        """
        if self.window != None:
            return
        window = video.SDL_CreateWindow(byteify(self._title, "utf-8"),
                                        self._position[0], self._position[1],
                                        self._size[0], self._size[1],
                                        self._flags)
        if not window:
            raise SDLError()
        self.window = window.contents

    def open(self):
        """Creates and shows the window."""
        if self.window is None:
            self.create()
        self.show()

    def close(self):
        """Closes the window, implicitly destroying the underlying SDL2
        window."""
        if hasattr(self, "window") and self.window != None:
            video.SDL_DestroyWindow(self.window)
            self.window = None

    def show(self):
        """Show the window on the display."""
        video.SDL_ShowWindow(self.window)

    def hide(self):
        """Hides the window."""
        video.SDL_HideWindow(self.window)

    def maximize(self):
        """Maximizes the window to the display's dimensions."""
        video.SDL_MaximizeWindow(self.window)

    def minimize(self):
        """Minimizes the window to an iconified state in the system tray."""
        video.SDL_MinimizeWindow(self.window)

    def refresh(self):
        """Refreshes the entire window surface.

        This only needs to be called, if a SDL_Surface was acquired via
        get_surface() and is used to display contents.
        """
        video.SDL_UpdateWindowSurface(self.window)

    def get_surface(self):
        """Gets the SDL_Surface used by the Window to display 2D pixel
        data.

        Using this method will make the usage of GL operations, such
        as texture handling, or using SDL renderers impossible.
        """
        sf = video.SDL_GetWindowSurface(self.window)
        if not sf:
            raise SDLError()
        return sf.contents



import sys
from ctypes import Structure, c_int, c_char_p, c_void_p, POINTER
from .dll import _bind
from .stdinc import Sint16, Sint32, Uint32, Uint16, Uint8, SDL_bool

__all__ = ["SDL_Joystick", "SDL_JoystickGUID", "SDL_JoystickID",
           "SDL_NumJoysticks", "SDL_JoystickNameForIndex", "SDL_JoystickOpen",
           "SDL_JoystickName", "SDL_JoystickGetDeviceGUID",
           "SDL_JoystickGetGUID", "SDL_JoystickGetGUIDString",
           "SDL_JoystickGetGUIDFromString", "SDL_JoystickGetAttached",
           "SDL_JoystickInstanceID", "SDL_JoystickNumAxes",
           "SDL_JoystickNumBalls", "SDL_JoystickNumHats",
           "SDL_JoystickNumButtons", "SDL_JoystickUpdate",
           "SDL_JoystickEventState", "SDL_JoystickGetAxis", "SDL_HAT_CENTERED",
           "SDL_HAT_UP", "SDL_HAT_RIGHT", "SDL_HAT_DOWN", "SDL_HAT_LEFT",
           "SDL_HAT_RIGHTUP", "SDL_HAT_RIGHTDOWN", "SDL_HAT_LEFTUP",
           "SDL_HAT_LEFTDOWN", "SDL_JoystickGetHat", "SDL_JoystickGetBall",
           "SDL_JoystickGetButton", "SDL_JoystickClose",
           "SDL_JOYSTICK_POWER_UNKNOWN", "SDL_JOYSTICK_POWER_EMPTY",
           "SDL_JOYSTICK_POWER_LOW", "SDL_JOYSTICK_POWER_MEDIUM",
           "SDL_JOYSTICK_POWER_FULL", "SDL_JOYSTICK_POWER_WIRED",
           "SDL_JOYSTICK_POWER_MAX", "SDL_JoystickPowerLevel",
           "SDL_JoystickCurrentPowerLevel", "SDL_JoystickFromInstanceID",
           "SDL_JoystickGetVendor", "SDL_JoystickGetProductVersion",
           "SDL_JoystickGetProduct", "SDL_JoystickGetAxisInitialState",
           "SDL_JoystickType", "SDL_JOYSTICK_TYPE_UNKNOWN",
           "SDL_JOYSTICK_TYPE_GAMECONTROLLER", "SDL_JOYSTICK_TYPE_WHEEL",
           "SDL_JOYSTICK_TYPE_ARCADE_STICK", "SDL_JOYSTICK_TYPE_FLIGHT_STICK",
           "SDL_JOYSTICK_TYPE_DANCE_PAD", "SDL_JOYSTICK_TYPE_GUITAR",
           "SDL_JOYSTICK_TYPE_DRUM_KIT", "SDL_JOYSTICK_TYPE_ARCADE_PAD",
           "SDL_JOYSTICK_TYPE_THROTTLE",
           "SDL_JoystickGetDeviceType", "SDL_JoystickGetType",
           "SDL_JoystickGetDeviceInstanceID", "SDL_LockJoysticks",
           "SDL_UnlockJoysticks", "SDL_JoystickGetPlayerIndex",
           "SDL_JoystickGetDevicePlayerIndex", "SDL_JoystickRumble"
          ]

SDL_JoystickPowerLevel = c_int

SDL_JOYSTICK_POWER_UNKNOWN = -1
SDL_JOYSTICK_POWER_EMPTY = 0
SDL_JOYSTICK_POWER_LOW = 1
SDL_JOYSTICK_POWER_MEDIUM = 2
SDL_JOYSTICK_POWER_FULL = 3
SDL_JOYSTICK_POWER_WIRED = 4
SDL_JOYSTICK_POWER_MAX = 5

SDL_JoystickType = c_int

SDL_JOYSTICK_TYPE_UNKNOWN = 0
SDL_JOYSTICK_TYPE_GAMECONTROLLER = 1
SDL_JOYSTICK_TYPE_WHEEL = 2
SDL_JOYSTICK_TYPE_ARCADE_STICK = 3
SDL_JOYSTICK_TYPE_FLIGHT_STICK = 4
SDL_JOYSTICK_TYPE_DANCE_PAD = 5
SDL_JOYSTICK_TYPE_GUITAR = 6
SDL_JOYSTICK_TYPE_DRUM_KIT = 7
SDL_JOYSTICK_TYPE_ARCADE_PAD = 8
SDL_JOYSTICK_TYPE_THROTTLE = 9


class SDL_Joystick(c_void_p):
    pass

class SDL_JoystickGUID(Structure):
    _fields_ = [("data", (Uint8 * 16))]

SDL_JoystickID = Sint32

SDL_NumJoysticks = _bind("SDL_NumJoysticks", None, c_int)
SDL_JoystickNameForIndex = _bind("SDL_JoystickNameForIndex", [c_int], c_char_p)
SDL_JoystickOpen = _bind("SDL_JoystickOpen", [c_int], POINTER(SDL_Joystick))
SDL_JoystickName = _bind("SDL_JoystickName", [POINTER(SDL_Joystick)], c_char_p)
SDL_JoystickGetDeviceGUID = _bind("SDL_JoystickGetDeviceGUID", [c_int], SDL_JoystickGUID)
SDL_JoystickGetGUID = _bind("SDL_JoystickGetGUID", [POINTER(SDL_Joystick)], SDL_JoystickGUID)
SDL_JoystickGetGUIDFromString = _bind("SDL_JoystickGetGUIDFromString", [c_char_p], SDL_JoystickGUID)
SDL_JoystickGetAttached = _bind("SDL_JoystickGetAttached", [POINTER(SDL_Joystick)], SDL_bool)
SDL_JoystickInstanceID = _bind("SDL_JoystickInstanceID", [POINTER(SDL_Joystick)], SDL_JoystickID)
SDL_JoystickNumAxes = _bind("SDL_JoystickNumAxes", [POINTER(SDL_Joystick)], c_int)
SDL_JoystickNumBalls = _bind("SDL_JoystickNumBalls", [POINTER(SDL_Joystick)], c_int)
SDL_JoystickNumHats = _bind("SDL_JoystickNumHats", [POINTER(SDL_Joystick)], c_int)
SDL_JoystickNumButtons = _bind("SDL_JoystickNumButtons", [POINTER(SDL_Joystick)], c_int)
SDL_JoystickUpdate = _bind("SDL_JoystickUpdate")
SDL_JoystickEventState = _bind("SDL_JoystickEventState", [c_int], c_int)
SDL_JoystickGetAxis = _bind("SDL_JoystickGetAxis", [POINTER(SDL_Joystick), c_int], Sint16)
SDL_HAT_CENTERED = 0x00
SDL_HAT_UP = 0x01
SDL_HAT_RIGHT = 0x02
SDL_HAT_DOWN = 0x04
SDL_HAT_LEFT = 0x08
SDL_HAT_RIGHTUP = SDL_HAT_RIGHT | SDL_HAT_UP
SDL_HAT_RIGHTDOWN = SDL_HAT_RIGHT | SDL_HAT_DOWN
SDL_HAT_LEFTUP = SDL_HAT_LEFT | SDL_HAT_UP
SDL_HAT_LEFTDOWN = SDL_HAT_LEFT | SDL_HAT_DOWN
SDL_JoystickGetHat = _bind("SDL_JoystickGetHat", [POINTER(SDL_Joystick), c_int], Uint8)
SDL_JoystickGetBall = _bind("SDL_JoystickGetBall", [POINTER(SDL_Joystick), c_int, POINTER(c_int), POINTER(c_int)], c_int)
SDL_JoystickGetButton = _bind("SDL_JoystickGetButton", [POINTER(SDL_Joystick), c_int], Uint8)
SDL_JoystickClose = _bind("SDL_JoystickClose", [POINTER(SDL_Joystick)])
SDL_JoystickCurrentPowerLevel = _bind("SDL_JoystickCurrentPowerLevel", [POINTER(SDL_Joystick)], SDL_JoystickPowerLevel)
SDL_JoystickFromInstanceID = _bind("SDL_JoystickFromInstanceID", [SDL_JoystickID], POINTER(SDL_Joystick))
SDL_JoystickGetVendor = _bind("SDL_JoystickGetVendor", [POINTER(SDL_Joystick)], Uint16, added='2.0.6')
SDL_JoystickGetProduct = _bind("SDL_JoystickGetProduct", [POINTER(SDL_Joystick)], Uint16, added='2.0.6')
SDL_JoystickGetProductVersion = _bind("SDL_JoystickGetProductVersion", [POINTER(SDL_Joystick)], Uint16, added='2.0.6')
SDL_JoystickGetAxisInitialState = _bind("SDL_JoystickGetAxisInitialState", [POINTER(SDL_Joystick), c_int, POINTER(Sint16)], SDL_bool, added='2.0.6')
SDL_JoystickGetType = _bind("SDL_JoystickGetType", [POINTER(SDL_Joystick)], SDL_JoystickType, added='2.0.6')
SDL_JoystickGetDeviceVendor = _bind("SDL_JoystickGetDeviceVendor", [c_int], Uint16, added='2.0.6')
SDL_JoystickGetDeviceProduct = _bind("SDL_JoystickGetDeviceProduct", [c_int], Uint16, added='2.0.6')
SDL_JoystickGetDeviceProductVersion = _bind("SDL_JoystickGetDeviceProductVersion", [c_int], Uint16, added='2.0.6')
SDL_JoystickGetDeviceType = _bind("SDL_JoystickGetDeviceType", [c_int], SDL_JoystickType, added='2.0.6')
SDL_JoystickGetDeviceInstanceID = _bind("SDL_JoystickGetDeviceInstanceID", [c_int], SDL_JoystickID, added='2.0.6')
SDL_LockJoysticks = _bind("SDL_LockJoysticks", None, None, added='2.0.7')
SDL_UnlockJoysticks = _bind("SDL_UnlockJoysticks", None, None, added='2.0.7')
SDL_JoystickGetPlayerIndex = _bind("SDL_JoystickGetPlayerIndex", [POINTER(SDL_Joystick)], c_int, added='2.0.9')
SDL_JoystickGetDevicePlayerIndex = _bind("SDL_JoystickGetDevicePlayerIndex", [c_int], c_int, added='2.0.9')
SDL_JoystickRumble = _bind("SDL_JoystickRumble", [POINTER(SDL_Joystick), Uint16, Uint16, Uint32], c_int, added='2.0.9')

# Reimplemented in Python due to crash-causing ctypes bug (fixed in 3.8)
if sys.version_info >= (3, 8, 0, 'final'):
    SDL_JoystickGetGUIDString = _bind("SDL_JoystickGetGUIDString", [SDL_JoystickGUID, c_char_p, c_int])
else:
    def SDL_JoystickGetGUIDString(guid, pszGUID, cbGUID):
        s = ""
        for g in guid.data:
            s += "{:x}".format(g >> 4)
            s += "{:x}".format(g & 0x0F)
        s = s.encode('utf-8')
        pszGUID.value = s[:(cbGUID * 2)]



import sys
import pytest
from sdl2 import ext as sdl2ext
from sdl2 import surface, video


class TestSDL2ExtWindow(object):
    __tags__ = ["sdl", "sdl2ext"]

    @classmethod
    def setup_class(cls):
        try:
            sdl2ext.init()
        except sdl2ext.SDLError:
            raise pytest.skip('Video subsystem not supported')

    @classmethod
    def teardown_class(cls):
        sdl2ext.quit()

    def test_Window(self):
        flags = video.SDL_WINDOW_BORDERLESS
        sizes = ((1, 1), (10, 10), (10, 20), (200, 17), (640, 480), (800, 600))
        for w, h in sizes:
            window = sdl2ext.Window("Window", size=(w, h), flags=flags)
            assert window.size == (w, h)
            window.close()

        with pytest.raises(TypeError):
            sdl2ext.Window("Test", None, None, None)
        with pytest.raises(TypeError):
            sdl2ext.Window("Test", None, None)
        with pytest.raises(TypeError):
            sdl2ext.Window("Test", None)


    def test_Window_title(self):
        window = sdl2ext.Window("Window", size=(10, 10))
        assert window.title == "Window"
        window.title = "Test1234"
        assert window.title == "Test1234"
        window.close()

        #window.title = None
        #self.assertEqual(window.title, "None")
        #window.title = 1234
        #self.assertEqual(window.title, "1234")

#     @interactive("Was the window shown?")
#     def test_Window_show(self):
#         window = sdl2ext.Window("Test Show Window", size=(200, 200))
#         window.show()
#         doprint("""Please check, if a window with the title
# 'Test Show Window' is shown""")

#     @interactive("Did the window vanish from your sight and pop up again?")
#     def test_Window_hide(self):
#         window = sdl2ext.Window("Test Hide Window", size=(200, 200))
#         window.show()
#         doprint("""Please check, if a window with the title
# 'Test Hide Window' is shown""")
#         window.hide()
#         doprint("Please check, that the window is not shown anymore")
#         window.show()
#         doprint("Please check, if the window is shown again")

#     @interactive("Was the window maximized?")
#     def test_Window_maximize(self):
#         window = sdl2ext.Window("Test Maximize Window", size=(200, 200),
#                                 flags=video.SDL_WINDOW_RESIZABLE)
#         window.show()
#         doprint("""Please check, that a window with the title
# 'Test Maximize Window' is shown""")
#         window.maximize()
#         doprint("Please check, if the window was maximized properly")

#     @interactive("Was the window minimized?")
#     def test_Window_minimize(self):
#         window = sdl2ext.Window("Test Minimize Window", size=(200, 200))
#         window.show()
#         doprint("""Please check, that a window with the title
# 'Test Minimize Window' is shown""")
#         window.minimize()
#         doprint("Please check, if the window was minimized properly")

    @pytest.mark.skip("not implemented")
    def test_Window_refresh(self):
        pass

    def test_Window_get_surface(self):
        window = sdl2ext.Window("Surface", size=(200, 200))
        sf = window.get_surface()
        assert isinstance(sf, surface.SDL_Surface)
        window.close()

    @pytest.mark.skip("not implemented")
    def test_Window_open(self):
        pass

    @pytest.mark.skip("not implemented")
    def test_Window_close(self):
        pass

    def test_Window_position(self):
        window = sdl2ext.Window("Position", size=(200, 200), position=(100, 100))
        assert window.position == (100, 100)
        window.position = 70, 300
        assert window.position == (70, 300)
        window.close()

    @pytest.mark.skip("not implemented")
    def test_Window_size(self):
        # This may fail for fullscreen WMs or Win10 tablet modes
        window = sdl2ext.Window("Size", size=(200, 200), flags=video.SDL_WINDOW_RESIZABLE)
        assert window.size == (200, 200)
        window.size = 150, 77
        assert window.size == (150, 77)
        window.close()



"""A simple example for filling rectangular areas."""
import sys

import sdl2
import sdl2.ext
import sdl2.ext.colorpalettes


# This function will draw the passed palette colors onto the window surface.
# The function does not require the window surface itself, any surface is fine,
# since the window surface acquired in run() does not differ from any other
# surface (not entirely true, but it offers and supports the same
# functionality).
def draw_palette(surface, palette):
    # Fill the entire surface with a black color. This is done by simply
    # passing a 0 value to the fill argument. We could also create a
    # Color(0, 0, 0) instance here, but this would be the same.
    sdl2.ext.fill(surface, 0)

    # Calculate the average width (roughly cut) to be used for each palette
    # value. When running the example, you will notice a black gap on the
    # right for some palettes. This s caused by the implicit cut behaviour
    # of the // operator. Since we can only operate pixel-wise, there are
    # no fractions to be used.
    width, height = surface.w, surface.h
    rw = width // len(palette)

    # Create the area to be filled with the palette values. we always start
    # at the top-left corner, use the calculated width and the entire height
    # of the window surface. As you will see below, we then only advance
    # horizontically by the calculated width to draw stripes.
    # Play around with different height values and start offsets to see what
    # happens
    rect = [0, 0, rw, height]

    # Loop over all colors and fill a portion of the surface with them. As
    # above, we use fill() to fill the surface with the palette color, but now
    # we provide an area (the third argument) to avoid filling the whole
    # surface. Instead, the provided area makes sure, that we only fill a
    # certain part.
    for color in palette:
        sdl2.ext.fill(surface, color, rect)
        rect[0] += rw


def run():
    # You know those from the helloworld.py example.
    # Initialize the video subsystem, create a window and make it visible.
    sdl2.ext.init()
    window = sdl2.ext.Window("Color Palettes", size=(800, 600))
    window.show()

    # Explicitly acquire the window's surface to draw on. We used the
    # SpriteRenderSystem class in helloworld.py, which did the drawing magic
    # for us. Now we will do it ourselves, so we have to get a surface to draw
    # on.
    #
    # NOTE: if you intend to use textures or the SDL renderers, you must not
    # use the method.
    windowsurface = window.get_surface()

    # A simple mapping table for the builtin color palettes. We will use
    # the table to look up the color palette to draw an the title to set below.
    palettes = (
        ("Mono Palette", sdl2.ext.colorpalettes.MONOPALETTE),
        ("2-bit Gray Palette", sdl2.ext.colorpalettes.GRAY2PALETTE),
        ("4-bit Gray Palette", sdl2.ext.colorpalettes.GRAY4PALETTE),
        ("8-bit Gray Palette", sdl2.ext.colorpalettes.GRAY8PALETTE),
        ("3-bit RGB Palette", sdl2.ext.colorpalettes.RGB3PALETTE),
        ("CGA Palette", sdl2.ext.colorpalettes.CGAPALETTE),
        ("EGA Palette", sdl2.ext.colorpalettes.EGAPALETTE),
        ("VGA Palette", sdl2.ext.colorpalettes.VGAPALETTE),
        ("Web Palette", sdl2.ext.colorpalettes.WEBPALETTE),
        )

    # A storage variable for the palette we are currently on, so that we know
    # which palette to draw next.
    curindex = 0

    # Since it is not that nice to have a black window right at the start of
    # the application, we will set the window's title to the first entry
    # of our mapping tables. Afterwards, we will draw the matching palette
    # to the window surface.
    window.title = palettes[0][0]
    draw_palette(windowsurface, palettes[0][1])

    # The event loop. In helloworld.py we used the TestEventProcessor class,
    # since there was not much to do. Now however, we want to react on the
    # user input. Every time the user clicks around in our window, we want to
    # show the next palette. Once we reached the last palette within the
    # mapping table, we will start again with the first one.
    running = True
    while running:
        # This will check for any events that piled up since the last check.
        # If one or multiple events were  found (such as a click, a mouse
        # movement, keyboard input, etc.), we will retrieve them.
        events = sdl2.ext.get_events()

        # In case there was no event, we do not need to do anything. This
        # might happen, if  e.g. the user works with another application. Since
        # get_events() does not wait for an event to occur (that'd mean your
        # application blocks until there is an event), we have to handle
        # this.
        for event in events:
            # The received events can contain different information. There
            # might be mouse movements, clicks, keyboard hits and many more.
            # All of those carry different information. A mouse movement will
            # contain the mouse cursor position, while a keyoard hit will
            # contain the key that was pressed. Depending on that, we need to
            # handle the occured event in a different way, which is done here.
            #
            # In case of a special QUIT event, the user wants to quit the
            # application, just as you are used to closing an editor.
            # If the user wants to quit the application, we should let him do
            # so. This is done by breaking out of the while loop.
            if event.type == sdl2.SDL_QUIT:
                running = False
                break

            # We received a mouse button press event. As you can see from the
            # type, the user pressed the mouse button, but did not necesarily
            # release  it. As such, it is not a typical click, but only 50% of
            # it, which is sufficient for our case here.
            if event.type == sdl2.SDL_MOUSEBUTTONDOWN:
                # If the user pressed the button, we want to draw the next
                # palette and update the window title accordingly. We do this
                # by increasing the storage variable and - in case it reached
                # the last entry, set it back to the first entry.
                curindex += 1
                if curindex >= len(palettes):
                    curindex = 0
                window.title = palettes[curindex][0]
                draw_palette(windowsurface, palettes[curindex][1])
                # If we found a single click (there might be many more)
                # we will break out of checking the rest of events.
                # Improved implementations could use the type= argument
                # for get_events() to filter specific events and
                # ignore anything else.
                break

        # Once the events were properly handled, we will refresh the window,
        # since it might have happened that the user moved the window around,
        # pressed a button or did something else. In all those cases, we want
        # the palettes to be shown, so we need to refresh the window. This will
        # cause the  window internally to copy its surface information (those
        # we used to draw the palette on) to the screen, where the window
        # currently is  placed on.
        # Comment this line out to see what happens!
        window.refresh()

    # As for helloworld.py, we have to call sdl2.ext.quit(), since we also
    # called sdl2.ext.init().
    sdl2.ext.quit()
    return 0


if __name__ == "__main__":
    sys.exit(run())



import sys
import pytest
from sdl2 import ext as sdl2ext
from sdl2 import surface

try:
    from sdl2 import sdlimage
    _HASSDLIMAGE=True
except:
    _HASSDLIMAGE=False

RESOURCES = sdl2ext.Resources(__file__, "resources")

is32bit = sys.maxsize <= 2**32
ismacos = sys.platform == "darwin"

formats = [ # Do not use bmp - it's contained in resources.zip
           "cur",
           "gif",
           "ico",
           "jpg",
           "lbm",
           "pbm",
           "pcx",
           "pgm",
           "png",
           "pnm",
           "ppm",
           "svg",
           "tga",
           "tif",
           "webp",
           "xcf",
           "xpm",
           # "xv",
           ]

# SVG unsupported on SDL2_image < 2.0.2
if _HASSDLIMAGE and sdlimage.dll.version < 2002:
    formats.remove("svg")

# As of SDL2_image 2.0.5, XCF support seems to be broken on 32-bit builds
# XCF support is also broken in official SDL2_image macOS .frameworks
if is32bit or ismacos:
    formats.remove("xcf")


class TestSDL2ExtImage(object):
    __tags__ = ["sdl", "sdl2ext"]

    @classmethod
    def setup_class(cls):
        try:
            sdl2ext.init()
        except sdl2ext.SDLError:
            raise pytest.skip('Video subsystem not supported')

    @classmethod
    def teardown_class(cls):
        sdl2ext.quit()

    def test_get_image_formats(self):
        assert isinstance(sdl2ext.get_image_formats(), tuple)
        supformats = sdl2ext.get_image_formats()
        for fmt in formats:
            assert fmt in supformats

    def test_load_image(self):
        # TODO: add image comparision to check, if it actually does the
        # right thing (SDL2 BMP loaded image?)
        # Add argument tests
        try:
            import PIL
            _HASPIL = True
        except ImportError:
            _HASPIL = False

        fname = "surfacetest.%s"
        for fmt in formats:
            filename = RESOURCES.get_path(fname % fmt)
            sf = sdl2ext.load_image(filename)
            assert isinstance(sf, surface.SDL_Surface)

            # Force only PIL
            if _HASPIL and fmt not in ("webp", "xcf", "lbm", "svg"):
                sf = sdl2ext.load_image(filename, enforce="PIL")
                assert isinstance(sf, surface.SDL_Surface)

            # Force only sdlimage
            sf = sdl2ext.load_image(filename, enforce="SDL")
            assert isinstance(sf, surface.SDL_Surface)



import sys
import pytest
from ctypes import c_int, cast, POINTER
from sdl2 import SDL_Init, SDL_Quit, SDL_QuitSubSystem, SDL_INIT_VIDEO
from sdl2 import rect, keyboard, scancode, keycode, video

byteify = lambda x: x.encode("utf-8")


class TestSDLKeyboard(object):
    __tags__ = ["sdl"]

    @classmethod
    def setup_class(cls):
        if SDL_Init(SDL_INIT_VIDEO) != 0:
            raise pytest.skip('Video subsystem not supported')

    @classmethod
    def teardown_class(cls):
        SDL_QuitSubSystem(SDL_INIT_VIDEO)
        SDL_Quit()

    def test_SDL_Keysym(self):
        keysym = keyboard.SDL_Keysym()
        assert keysym.scancode == 0
        assert keysym.sym == 0
        assert keysym.mod == 0

        keysym = keyboard.SDL_Keysym(1, 2, 3, ord("b"))
        assert keysym.scancode == 1
        assert keysym.sym == 2
        assert keysym.mod == 3

        uval = "\u0220"
        if sys.version_info[0] < 3:
            uval = unichr(8224)
        keysym = keyboard.SDL_Keysym(17, 32, 88, ord(uval))
        assert keysym.scancode == 17
        assert keysym.sym == 32
        assert keysym.mod == 88

    @pytest.mark.skipif(sys.platform == "darwin", reason="crashes on OS X")
    def test_SDL_GetKeyboardFocus(self):
        # window = keyboard.SDL_GetKeyboardFocus()
        # TODO: x
        # self.assertEqual(window, None)
        rwin = video.SDL_CreateWindow(b"test", 10, 10, 10, 10, 0)
        window = keyboard.SDL_GetKeyboardFocus()
        if window:
            assert video.SDL_GetWindowID(window) == video.SDL_GetWindowID(rwin)
        video.SDL_DestroyWindow(rwin)

        window = keyboard.SDL_GetKeyboardFocus()
        # TODO: x
        # self.assertEqual(window, None)

    def test_SDL_GetKeyboardState(self):
        states = (c_int * scancode.SDL_NUM_SCANCODES)()
        keyboard.SDL_GetKeyboardState(cast(states, POINTER(c_int)))
        assert len(states) == scancode.SDL_NUM_SCANCODES
        # TODO: x
        # for state in states:
        #    self.assertEqual(state, 0)

    def test_SDL_GetKeyFromName(self):
        for x in range(26):  # a-z
            key = keyboard.SDL_GetKeyFromName(byteify(chr(x + 97)))
            assert key == x + 97

        for x in range(10):
            key = keyboard.SDL_GetKeyFromName(("%d" % x).encode("utf-8"))
            assert key == 48 + x

        val = keyboard.SDL_GetKeyFromName(byteify(repr(self)))
        assert val == keycode.SDLK_UNKNOWN

    def test_SDL_GetKeyFromScancode(self):
        p = 0
        for x in (scancode.SDL_SCANCODE_A,
                  scancode.SDL_SCANCODE_B,
                  scancode.SDL_SCANCODE_C):
            key = keyboard.SDL_GetKeyFromScancode(x)
            assert key == p + 97
            p += 1

        p = 0
        for x in range(scancode.SDL_SCANCODE_1, scancode.SDL_SCANCODE_0):
            key = keyboard.SDL_GetKeyFromScancode(x)
            assert key == 49 + p
            p += 1

    def test_SDL_GetKeyName(self):
        x = 65  # SDL maps everything against upper-case letters
        for key in range(ord('a'), ord('z')):
            ch = chr(x)
            name = keyboard.SDL_GetKeyName(key)
            assert name == byteify(ch)
            x += 1

    def test_SDL_GetSetModState(self):
        initial = keyboard.SDL_GetModState()
        for state in(keycode.KMOD_NUM | keycode.KMOD_CAPS | keycode.KMOD_MODE,
                      keycode.KMOD_NUM | keycode.KMOD_CAPS,
                      keycode.KMOD_CAPS):
            keyboard.SDL_SetModState(state)
            assert keyboard.SDL_GetModState() == state

        state = keycode.KMOD_NUM
        keyboard.SDL_SetModState(state)
        assert keyboard.SDL_GetModState() == state

        keyboard.SDL_SetModState(initial)
        assert keyboard.SDL_GetModState() == initial

    def test_SDL_GetScancodeFromKey(self):
        codes = range(scancode.SDL_SCANCODE_1, scancode.SDL_SCANCODE_0)
        xoff = 0
        for key in range(ord('1'), ord('0')):
            code = keyboard.SDL_GetScancodeFromKey(key)
            assert code == codes[xoff]
            xoff += 1

        key = keyboard.SDL_GetScancodeFromKey(477)
        assert key == scancode.SDL_SCANCODE_UNKNOWN

    def test_SDL_GetScancodeFromName(self):
        codes = range(scancode.SDL_SCANCODE_A, scancode.SDL_SCANCODE_Z)
        xoff = 0
        for key in range(ord('a'), ord('z')):
            ch = chr(key)
            code = keyboard.SDL_GetScancodeFromName(byteify(ch))
            assert code == codes[xoff]
            xoff += 1

        key = keyboard.SDL_GetScancodeFromName(b"")
        assert key == scancode.SDL_SCANCODE_UNKNOWN

        key = keyboard.SDL_GetScancodeFromName(None)
        assert key == scancode.SDL_SCANCODE_UNKNOWN

        key = keyboard.SDL_GetScancodeFromName(b"Test")
        assert key == scancode.SDL_SCANCODE_UNKNOWN

        key = keyboard.SDL_GetScancodeFromName(byteify(repr(self)))
        assert key == scancode.SDL_SCANCODE_UNKNOWN

    def test_SDL_GetScancodeName(self):
        names = range(ord('A'), ord('Z'))
        xoff = 0
        for code in range(scancode.SDL_SCANCODE_A, scancode.SDL_SCANCODE_Z):
            name = keyboard.SDL_GetScancodeName(code)
            assert name == byteify(chr(names[xoff]))
            xoff += 1

        name = keyboard.SDL_GetScancodeName(0)
        assert name == b""

        # self.assertRaises(ValueError, keyboard.SDL_GetScancodeName, -22)
        # self.assertRaises(ValueError, keyboard.SDL_GetScancodeName,
        #                   scancode.SDL_NUM_SCANCODES)

    def test_SDL_SetTextInputRect(self):
        # TODO: this test is a bit pointless
        coords = [(0, 0, 0, 0), (-10, -70, 3, 6), (10, 10, 10, 10)]
        for x, y, w, h in coords:
            r = rect.SDL_Rect(x, y, w, h)
            keyboard.SDL_SetTextInputRect(r)
        keyboard.SDL_SetTextInputRect(rect.SDL_Rect())

    @pytest.mark.skip("not implemented")
    def test_SDL_StartTextInput(self):
        keyboard.SDL_StartTextInput()

    @pytest.mark.skip("not implemented")
    def test_SDL_StopTextInput(self):
        keyboard.SDL_StopTextInput()
